CREATE OR REPLACE PACKAGE BODY PK_ABM
AS
   /**
   * Estat del Parlament.
   * <p>Funció interna que mira si el parlament existeix i si esta operatiu,
   * @param p_id                 NUMBER. OBLIGATORI. Codi identificatiu del parlament
   *
   * @return                     BOOLEAN. TRUE = El parlament esta operatiu; FALSE = Parlament no existeix o esta donat de baixa
	*/
   FUNCTION isParlamentOperatiu (
      p_id           IN    parlament.id%TYPE
   )
      RETURN BOOLEAN
   IS 
      v_exist     NUMBER;
   BEGIN
      SELECT 1 INTO v_exist
        FROM parlament
       WHERE id = p_id 
         AND databaixa IS NULL;
      
      RETURN TRUE;
   EXCEPTION
      WHEN NO_DATA_FOUND
      THEN
         RETURN FALSE;
   END isParlamentOperatiu;

   /**
   * Estat del Partit Politic.
   * <p>Funció interna que mira si el partit politic existeix i si esta operatiu,
   * @param p_id                 NUMBER. OBLIGATORI. Codi identificatiu del partit politic
   *
   * @return                     BOOLEAN. TRUE = El partit esta operatiu; FALSE = El partit no existeix o esta donat de baixa
	*/    
   FUNCTION isPartitOperatiu (
      p_id           IN    partit.id%TYPE
   )
      RETURN BOOLEAN
   IS 
      v_exist     NUMBER;
   BEGIN
      SELECT 1 INTO v_exist
        FROM partit
       WHERE id = p_id 
         AND databaixa IS NULL;
      
      RETURN TRUE;
   EXCEPTION
      WHEN NO_DATA_FOUND
      THEN
         RETURN FALSE;
   END isPartitOperatiu;   
   
      /**
   * Estat del Parlamentari.
   * <p>Funció interna que mira si el parlamentari existeix i si esta operatiu,
   * @param p_id                 NUMBER. OBLIGATORI. Codi identificatiu del parlamentari
   *
   * @return                     BOOLEAN. TRUE = El parlamentari esta operatiu; FALSE = El parlamentari no existeix o esta donat de baixa
	*/    
   FUNCTION isParlamentariOperatiu (
      p_id           IN    parlamentari.id%TYPE
   )
      RETURN BOOLEAN
   IS
      v_exist     NUMBER;
   BEGIN
      SELECT 1 INTO v_exist
        FROM parlamentari
       WHERE id = p_id
         AND databaixa IS NULL;
      
      RETURN TRUE;
   EXCEPTION
      WHEN NO_DATA_FOUND
      THEN
         RETURN FALSE;
   END isParlamentariOperatiu;
   
	PROCEDURE alta_parlament (
		p_document		IN	   parlament.document%TYPE,
		p_nom       	IN 	parlament.nom%TYPE,
		p_pais      	IN    parlament.pais%TYPE,
      p_regio        IN    parlament.regio%TYPE,
		p_direccio  	IN    parlament.direccio%TYPE,
		p_telefon   	IN    parlament.telefon%TYPE,
		p_web       	IN	   parlament.web%TYPE,
		p_usuari		   IN    parlament.usualta%TYPE,
		RSP				OUT	NOCOPY VARCHAR2
	) IS
		v_nomproc	      logs.nomproc%TYPE := 'PK_ABM.alta_parlament';
		v_parin		      logs.parin%TYPE;
		
      v_exist           NUMBER;
      
      ERR_PARAMS_NULL   EXCEPTION;
      ERR_PAIS          EXCEPTION;
      ERR_USER          EXCEPTION;
      ERR_DUP_PARLAMENT EXCEPTION;
      ERR_REGIO         EXCEPTION;
      ERR_PAISREGIO_PARLAMENT EXCEPTION;
      
	BEGIN
      v_parin := CONCAT ('p_document=', p_document);		
      v_parin := v_parin || CONCAT ('; p_nom=', p_nom);    
      v_parin := v_parin || CONCAT ('; p_pais=', p_pais);    
      v_parin := v_parin || CONCAT ('; p_regio=', p_regio);    
      v_parin := v_parin || CONCAT ('; p_direccio=', p_direccio);    
      v_parin := v_parin || CONCAT ('; p_telefon=', p_telefon);    
      v_parin := v_parin || CONCAT ('; p_web=', p_web);    
      v_parin := v_parin || CONCAT ('; p_usuari=', p_usuari);		
		
		BEGIN
         -- Validació del paràmetres d'entrada
         IF p_document IS NULL OR p_nom IS NULL OR p_pais IS NULL
            OR p_direccio IS NULL OR p_telefon IS NULL
         THEN
            RAISE ERR_PARAMS_NULL;
         END IF;
         
         IF p_usuari IS NULL 
         THEN
            RAISE ERR_USER;
         END IF;
         
         -- Comprova que el pais existeix
         SELECT COUNT(1) INTO v_exist
           FROM pais
          WHERE codi = p_pais;
         
         IF v_exist = 0
         THEN
            RAISE ERR_PAIS;
         END IF;
         
         -- En el cas que s'informi una regió, comprova que la regió
         -- esta assignada ala pais
         IF p_regio IS NOT NULL
         THEN
            SELECT COUNT(1) INTO v_exist
              FROM regio
             WHERE codi = p_regio
               AND pais = p_pais;
            
            IF v_exist = 0
            THEN
               RAISE ERR_REGIO;
            END IF;
         END IF;
         
         -- Comprova que no existeixi un parlament amb el mateix codi de país i regió
         SELECT COUNT(1) INTO v_exist
           FROM parlament
          WHERE pais = p_pais
            AND regio = p_regio;
         
         IF v_exist != 0 
         THEN
            RAISE ERR_PAISREGIO_PARLAMENT;
         END IF;
         
         -- Comprova que no existeixi un parlament amb el document identificatiu
         SELECT COUNT(1) INTO v_exist
           FROM parlament
          WHERE document = UPPER(p_document);
          
         IF v_exist != 0 
         THEN
            RAISE ERR_DUP_PARLAMENT;
         END IF;
         	
			INSERT INTO parlament (id, document, nom, pais, regio, direccio, 
								   telefon, web, usualta, dataalta)
				VALUES (seq_parlament.NEXTVAL, UPPER(p_document), p_nom, p_pais, p_regio, p_direccio, 
						p_telefon, p_web, p_usuari, sysdate);
			COMMIT;
			RSP :=  'OK';
		EXCEPTION
         WHEN ERR_PARAMS_NULL
         THEN
            RSP := PK_ERRORS.ERR_PARAMS_NULL;            
         WHEN ERR_USER
         THEN
            RSP := PK_ERRORS.ERR_USER;
         WHEN ERR_PAIS
         THEN
            RSP := PK_ERRORS.ERR_PAIS;
         WHEN ERR_DUP_PARLAMENT
         THEN
            RSP := PK_ERRORS.ERR_DUP_PARLAMENT;
         WHEN ERR_REGIO
         THEN
            RSP := PK_ERRORS.ERR_REGIO;
         WHEN ERR_PAISREGIO_PARLAMENT
         THEN
            RSP := PK_ERRORS.ERR_PAISREGIO_PARLAMENT;
			WHEN OTHERS 
         THEN
           RSP := CONCAT(PK_ERRORS.ERR_NOTRACTAT, (TO_CHAR(SQLCODE) || ' '|| SUBSTR(SQLERRM,1,400)));
			  ROLLBACK;
		END;
		p_log (v_nomproc, v_parin, RSP);
	END alta_parlament;
	
	
	PROCEDURE modif_parlament (
		p_id			   IN    parlament.id%TYPE,
		p_document		IN	   parlament.document%TYPE,
		p_nom       	IN    parlament.nom%TYPE,
		p_pais      	IN    parlament.pais%TYPE,
      p_regio        IN    parlament.regio%TYPE,
		p_direccio  	IN    parlament.direccio%TYPE,
		p_telefon   	IN    parlament.telefon%TYPE,
		p_web       	IN	   parlament.web%TYPE,
		p_usuari		   IN	   parlament.usumod%TYPE,
		RSP				OUT   NOCOPY VARCHAR2
	) IS
		v_nomproc	logs.nomproc%TYPE := 'PK_ABM.modif_parlament';
		v_parin		logs.parin%TYPE;
      
      v_exist              NUMBER;
      
      ERR_PARAMS_NULL         EXCEPTION;
      ERR_USER                EXCEPTION;
      ERR_PAIS                EXCEPTION;
      ERR_DUP_PARLAMENT       EXCEPTION;
      ERR_REGIO               EXCEPTION;
      ERR_PAISREGIO_PARLAMENT EXCEPTION;
      ERR_BAIXA_PARLAMENT     EXCEPTION;
	BEGIN
      v_parin := CONCAT('p_id=', p_id);
      v_parin := v_parin || CONCAT('; p_document=', p_document);
      v_parin := v_parin || CONCAT('; p_nom=', p_nom);
      v_parin := v_parin || CONCAT('; p_pais=', p_pais);
      v_parin := v_parin || CONCAT('; p_regio=', p_regio);
      v_parin := v_parin || CONCAT('; p_direccio=', p_direccio);
      v_parin := v_parin || CONCAT('; p_telefon=', p_telefon);
      v_parin := v_parin || CONCAT('; p_web=', p_web);
      v_parin := v_parin || CONCAT('; p_usuari=', p_usuari);

		BEGIN
         -- Validació del paràmetres d'entrada
         IF p_id IS NULL OR p_document IS NULL OR p_nom IS NULL 
            OR p_pais IS NULL OR p_direccio IS NULL OR p_telefon IS NULL
         THEN
            RAISE ERR_PARAMS_NULL;
         END IF;
         
         IF p_usuari IS NULL
         THEN
            RAISE ERR_USER;
         END IF;
         
         -- Comprova que el parlament estigui operatiu
         IF NOT isParlamentOperatiu (p_id)
         THEN
            RAISE ERR_BAIXA_PARLAMENT;
         END IF;
         
         -- Comprova que el pais del parlament estigui codificat
         SELECT COUNT(1) INTO v_exist
           FROM pais
          WHERE codi = p_pais;
          
         IF v_exist = 0
         THEN
            RAISE ERR_PAIS;
         END IF;
         
         -- Si la regió esta informada, mira que aquesta pertanyi al pais informat
         IF p_regio IS NOT NULL
         THEN
            SELECT COUNT(1) INTO v_exist
              FROM regio
             WHERE codi = p_regio
               AND pais = p_pais;
               
            IF v_exist = 0
            THEN
               RAISE ERR_REGIO;
            END IF;
         END IF;

         -- Comprova que no hi hagi un altre parlament amb el mateix pais i regio
         SELECT COUNT(1) INTO v_exist
           FROM parlament
          WHERE pais = p_pais
            AND regio = p_regio
            AND id != p_id;
         
         IF v_exist != 0 THEN
            RAISE ERR_PAISREGIO_PARLAMENT;
         END IF;
         
         -- Mira que no hi hagi un altre parlament amb el mateix document
         SELECT COUNT(1) INTO v_exist
           FROM parlament
          WHERE document = UPPER(p_document)
            AND id != p_id;
         
         IF v_exist != 0
         THEN
            RAISE ERR_DUP_PARLAMENT;
         END IF;
         
			UPDATE parlament
			   SET document = UPPER(p_document),
			       nom = p_nom,
			       pais = p_pais,
                regio = p_regio,
			       direccio = p_direccio,
			       telefon = p_telefon,
			       web = p_web,
			       usumod = p_usuari,
			       datamod = sysdate
			 WHERE id = p_id;
          
			COMMIT;
			RSP :=  'OK';
		EXCEPTION
         WHEN ERR_PARAMS_NULL
         THEN
            RSP := PK_ERRORS.ERR_PARAMS_NULL;
         WHEN ERR_USER
         THEN
            RSP := PK_ERRORS.ERR_USER;
         WHEN ERR_PAIS
         THEN
            RSP := PK_ERRORS.ERR_PAIS;
         WHEN ERR_DUP_PARLAMENT
         THEN
            RSP := PK_ERRORS.ERR_DUP_PARLAMENT;
         WHEN ERR_REGIO
         THEN
            RSP := PK_ERRORS.ERR_REGIO;
         WHEN ERR_PAISREGIO_PARLAMENT
         THEN
            RSP := PK_ERRORS.ERR_PAISREGIO_PARLAMENT;
         WHEN ERR_BAIXA_PARLAMENT
         THEN
            RSP := PK_ERRORS.ERR_BAIXA_PARLAMENT;
			WHEN OTHERS THEN
			  RSP := CONCAT(PK_ERRORS.ERR_NOTRACTAT, (TO_CHAR(SQLCODE) || ' '|| SUBSTR(SQLERRM,1,400)));
			  ROLLBACK;
		END;
		p_log (v_nomproc, v_parin, RSP);
	END modif_parlament;
	
	PROCEDURE baixa_parlament (
		p_id			   IN	   parlament.id%TYPE,
		p_usuari		   IN	   parlament.usubaixa%TYPE,
		RSP				OUT	NOCOPY VARCHAR2
	) IS
		v_nomproc	logs.nomproc%TYPE := 'PK_ABM.baixa_parlament';
		v_parin		logs.parin%TYPE;
      
      v_exist        NUMBER;
      
      ERR_PARAMS_NULL      EXCEPTION;
      ERR_USER             EXCEPTION;
      ERR_PARLA_PARLAMENT  EXCEPTION;
      ERR_BAIXA_PARLAMENT  EXCEPTION;
	BEGIN
		v_parin := CONCAT('p_id=', p_id);
		v_parin := v_parin || CONCAT('; p_usuari=', p_usuari);
      
		BEGIN
         -- Validació dels paràmetres d'entrada
         IF p_id IS NULL
         THEN
            RAISE ERR_PARAMS_NULL;
         END IF;
         
         IF p_usuari IS NULL
         THEN
            RAISE ERR_USER;
         END IF;
         
         -- Mira que el parlament no tengui parlamentaris operatius assignats
         SELECT COUNT(1) INTO v_exist
           FROM parlamentari
          WHERE parlament = p_id
            AND databaixa IS NULL;
         
         IF v_exist > 0
         THEN
            RAISE ERR_PARLA_PARLAMENT;
         END IF;
         
         -- Comprova que el parlament estigui operatiu
         IF NOT isParlamentOPeratiu(p_id)
         THEN
            RAISE ERR_BAIXA_PARLAMENT;
         END IF;
         
			UPDATE parlament
			   SET usubaixa = p_usuari,
				   databaixa = sysdate
			 WHERE id = p_id;
          
			COMMIT;
			RSP :=  'OK';
		EXCEPTION
         WHEN ERR_PARAMS_NULL
         THEN
            RSP := PK_ERRORS.ERR_PARAMS_NULL;
         WHEN ERR_USER
         THEN
            RSP := PK_ERRORS.ERR_USER;
         WHEN ERR_PARLA_PARLAMENT
         THEN
            RSP := PK_ERRORS.ERR_PARLA_PARLAMENT;
         WHEN ERR_BAIXA_PARLAMENT
         THEN
            RSP := PK_ERRORS.ERR_BAIXA_PARLAMENT;
			WHEN OTHERS THEN
			  RSP := CONCAT(PK_ERRORS.ERR_NOTRACTAT, (TO_CHAR(SQLCODE) || ' '|| SUBSTR(SQLERRM,1,400)));
			  ROLLBACK;
		END;
		p_log (v_nomproc, v_parin, RSP);
	END baixa_parlament;
   
   PROCEDURE react_parlament (
      p_id           IN    parlament.id%TYPE,
      p_usuari       IN    parlament.usumod%TYPE,
      RSP            OUT   NOCOPY VARCHAR2
   ) 
   IS
      v_nomproc	   logs.nomproc%TYPE := 'PK_ABM.react_parlament';
		v_parin		   logs.parin%TYPE;
      
      v_databaixa    parlament.databaixa%TYPE;   
      
      ERR_PARAMS_NULL         EXCEPTION;
      ERR_USER                EXCEPTION;
      ERR_PARLAMENT_OPERATIU  EXCEPTION;
      ERR_PARLAMENT_NEXIST    EXCEPTION;
   BEGIN
      v_parin := CONCAT ('p_id=', p_id);
      v_parin := v_parin || CONCAT ('; p_usuari=', p_usuari);
      
      BEGIN
         -- Validació dels paràmetres d'entrada
         IF p_id IS NULL
         THEN
            RAISE ERR_PARAMS_NULL;
         END IF;
         
         IF p_usuari IS NULL
         THEN
            RAISE ERR_USER;
         END IF;
         
         -- Recupera la data de baixa del parlament per validar que aquest
         -- existeix i que està donat de baixa
         BEGIN
            SELECT databaixa INTO v_databaixa
              FROM parlament
             WHERE id = p_id;
         EXCEPTION 
            WHEN NO_DATA_FOUND
            THEN
               RAISE ERR_PARLAMENT_NEXIST;
         END;
         
         IF v_databaixa IS NULL 
         THEN
            RAISE ERR_PARLAMENT_OPERATIU;
         ELSE
            UPDATE parlament 
               SET datamod = sysdate,
                   usumod = p_usuari,
                   databaixa = NULL,
                   usubaixa = NULL
             WHERE id = p_id;
         END IF;
         
         COMMIT;
         RSP := 'OK';
      EXCEPTION
         WHEN ERR_PARAMS_NULL
         THEN
            RSP := PK_ERRORS.ERR_PARAMS_NULL;
         WHEN ERR_USER
         THEN
            RSP := PK_ERRORS.ERR_USER;
         WHEN ERR_PARLAMENT_NEXIST
         THEN
            RSP := PK_ERRORS.ERR_PARLAMENT_NEXIST;
         WHEN ERR_PARLAMENT_OPERATIU
         THEN
            RSP := PK_ERRORS.ERR_PARLAMENT_OPERATIU;
         WHEN OTHERS THEN
			  RSP := CONCAT(PK_ERRORS.ERR_NOTRACTAT, (TO_CHAR(SQLCODE) || ' '|| SUBSTR(SQLERRM,1,400)));
			  ROLLBACK;
		END;
      p_log (v_nomproc, v_parin, RSP);
   END react_parlament;
	
	PROCEDURE alta_partit (
		p_document		IN	   partit.document%TYPE,
		p_nom			   IN	   partit.nom%TYPE,
		p_usuari		   IN	   partit.usualta%TYPE,
		RSP				OUT	NOCOPY VARCHAR2
	) IS
		v_nomproc	   logs.nomproc%TYPE := 'PK_ABM.alta_partit';
		v_parin		   logs.parin%TYPE;
		
      v_exist        NUMBER;
      
      ERR_PARAMS_NULL      EXCEPTION;
      ERR_USER             EXCEPTION;
      ERR_DUP_PARTIT       EXCEPTION;
      
	BEGIN
		v_parin := CONCAT('p_document=', p_document);
		v_parin := v_parin || CONCAT('; p_nom=', p_nom);
		v_parin := v_parin || CONCAT('; p_usuari=', p_usuari);
		
		BEGIN
         -- Validació dels paràmetres d'entrada
         IF p_document IS NULL OR p_nom IS NULL
         THEN
            RAISE ERR_PARAMS_NULL;
         END IF;
         
         IF p_usuari IS NULL
         THEN
            RAISE ERR_USER;
         END IF;
         
         -- Comprova que no existeix un altre partit amb el mateix document
         SELECT COUNT(1) INTO v_exist
           FROM partit
          WHERE document = UPPER(p_document);
          
         IF v_exist != 0 THEN
            RAISE ERR_DUP_PARTIT;
         END IF;
      		
			INSERT INTO partit (id, document, nom, usualta, dataalta)
				VALUES (seq_partit.NEXTVAL, UPPER(p_document), p_nom, p_usuari, sysdate);
			COMMIT;
			RSP :=  'OK';
      EXCEPTION
         WHEN ERR_PARAMS_NULL 
         THEN
            RSP := PK_ERRORS.ERR_PARAMS_NULL;
         WHEN ERR_USER 
         THEN
            RSP := PK_ERRORS.ERR_USER;
         WHEN ERR_DUP_PARTIT
         THEN
            RSP := PK_ERRORS.ERR_DUP_PARTIT;
         WHEN OTHERS 
         THEN
            RSP := CONCAT(PK_ERRORS.ERR_NOTRACTAT, (TO_CHAR(SQLCODE) || ' '|| SUBSTR(SQLERRM,1,400)));
            ROLLBACK;
		END;
		p_log (v_nomproc, v_parin, RSP);
	END alta_partit;
	
	PROCEDURE modif_partit (
		p_id			   IN 	partit.id%TYPE,
		p_document		IN	   partit.document%TYPE,
		p_nom			   IN 	partit.nom%TYPE,
		p_usuari		   IN	   partit.usumod%TYPE,
		RSP				OUT	NOCOPY VARCHAR2
	) IS
		v_nomproc	      logs.nomproc%TYPE := 'PK_ABM.modif_partit';
		v_parin		      logs.parin%TYPE;
      
      v_exist           NUMBER;
      
      ERR_PARAMS_NULL   EXCEPTION;
      ERR_USER          EXCEPTION;
      ERR_DUP_PARTIT    EXCEPTION;      
      ERR_BAIXA_PARTIT  EXCEPTION;
	BEGIN
		v_parin := CONCAT('p_id=', p_id);
		v_parin := v_parin || CONCAT('; p_document=', p_document);
		v_parin := v_parin || CONCAT('; p_nom=', p_nom);
		v_parin := v_parin || CONCAT('; p_usuari=', p_usuari);

		BEGIN
         -- Validació dels paràmetres d'entrada
         IF p_id IS NULL OR p_document IS NULL OR p_nom IS NULL
         THEN
            RAISE ERR_PARAMS_NULL;
         END IF;
         
         IF p_usuari IS NULL
         THEN
            RAISE ERR_USER;
         END IF;
         
         -- Comprova que no existeixi un altre partit amb el mateix document
         SELECT COUNT(1) INTO v_exist
           FROM partit
          WHERE document = UPPER(p_document)
            AND id != p_id;
         
         IF v_exist != 0 
         THEN
            RAISE ERR_DUP_PARTIT;
         END IF;
         
         -- Mira que el partit estigui operatiu
         IF NOT isPartitOperatiu(p_id)
         THEN
            RAISE ERR_BAIXA_PARTIT;
         END IF;
         
			UPDATE partit
			   SET document = UPPER(p_document),
			       nom = p_nom,
			       usumod = p_usuari,
			       datamod = sysdate
			 WHERE id = p_id;
			 
			COMMIT;
			RSP :=  'OK';
		EXCEPTION
         WHEN ERR_PARAMS_NULL
         THEN
            RSP := PK_ERRORS.ERR_PARAMS_NULL;
         WHEN ERR_USER
         THEN
            RSP := PK_ERRORS.ERR_USER;
         WHEN ERR_DUP_PARTIT
         THEN
            RSP := PK_ERRORS.ERR_DUP_PARTIT;
         WHEN ERR_BAIXA_PARTIT
         THEN
            RSP := PK_ERRORS.ERR_BAIXA_PARTIT;
			WHEN OTHERS THEN
			  RSP := CONCAT(PK_ERRORS.ERR_NOTRACTAT, (TO_CHAR(SQLCODE) || ' '|| SUBSTR(SQLERRM,1,400)));
			  ROLLBACK;
		END;
		p_log (v_nomproc, v_parin, RSP);
	END modif_partit;
	
	PROCEDURE baixa_partit (
		p_id			   IN	   partit.id%TYPE,
		p_usuari		   IN	   partit.usubaixa%TYPE,
		RSP				OUT	NOCOPY VARCHAR2
	) IS
		v_nomproc	         logs.nomproc%TYPE := 'PK_ABM.baixa_partit';
		v_parin		         logs.parin%TYPE;
      
      v_exist              NUMBER;
      
      ERR_PARAMS_NULL      EXCEPTION;
      ERR_USER             EXCEPTION;
      ERR_PARLA_PARTIT     EXCEPTION;
      ERR_BAIXA_PARTIT     EXCEPTION;
	BEGIN
		v_parin := CONCAT('p_id=', p_id);
		v_parin := v_parin || CONCAT('; p_usuari=', p_usuari);
      
		BEGIN
         -- Valiació dels paràmetres d'entrada
         IF p_id IS NULL
         THEN
            RAISE ERR_PARAMS_NULL;
         END IF;
         
         IF p_usuari IS NULL
         THEN
            RAISE ERR_USER;
         END IF;
         
         -- Mira si hi ha parlamentaris operatius al partit
         SELECT COUNT(1) INTO v_exist
           FROM parlamentari
          WHERE partit = p_id
            AND databaixa IS NULL;
          
         IF v_exist > 0 
         THEN
            RAISE ERR_PARLA_PARTIT;
         END IF;
         
         -- Comprova que el partit està operatiu
         IF NOT isPartitOperatiu (p_id)
         THEN
            RAISE ERR_BAIXA_PARTIT;
         END IF;
         
			UPDATE partit
			   SET usubaixa = p_usuari,
			       databaixa = sysdate
			 WHERE id = p_id;
			COMMIT;
			RSP :=  'OK';
		EXCEPTION
         WHEN ERR_PARAMS_NULL
         THEN
            RSP := PK_ERRORS.ERR_PARAMS_NULL;
         WHEN ERR_USER
         THEN
            RSP := PK_ERRORS.ERR_USER;
         WHEN ERR_PARLA_PARTIT
         THEN
            RSP := PK_ERRORS.ERR_PARLA_PARTIT;
         WHEN ERR_BAIXA_PARTIT
         THEN
            RSP := PK_ERRORS.ERR_BAIXA_PARTIT;
			WHEN OTHERS 
         THEN
			  RSP := CONCAT(PK_ERRORS.ERR_NOTRACTAT, (TO_CHAR(SQLCODE) || ' '|| SUBSTR(SQLERRM,1,400)));
			  ROLLBACK;
		END;
		p_log (v_nomproc, v_parin, RSP);
	END baixa_partit;
	
   PROCEDURE react_partit (
      p_id           IN    partit.id%TYPE,
      p_usuari       IN    partit.usumod%TYPE,
      RSP            OUT   NOCOPY VARCHAR2
   ) IS
      v_nomproc	            logs.nomproc%TYPE := 'PK_ABM.react_partit';
		v_parin		            logs.parin%TYPE;
      
      v_databaixa             partit.databaixa%TYPE;
      
      ERR_PARAMS_NULL         EXCEPTION;
      ERR_USER                EXCEPTION;
      ERR_PARTIT_NEXIST       EXCEPTION;
      ERR_PARTIT_OPERATIU     EXCEPTION;
   BEGIN
      v_parin := CONCAT ('p_id=',p_id);
      v_parin := v_parin || concat (' p_usuari=', p_usuari);
      
      BEGIN
         -- Validació dels paràmetres d'entrada
         IF p_id IS NULL
         THEN
            RAISE ERR_PARAMS_NULL;
         END IF;
         
         IF p_usuari IS NULL
         THEN
            RAISE ERR_USER;
         END IF;
         
         -- Recupera la data de baixa per validar que el partit existeix
         -- i que està donat de baixa
         BEGIN
            SELECT databaixa INTO v_databaixa
              FROM partit
             WHERE id = p_id;
         EXCEPTION 
            WHEN NO_DATA_FOUND
            THEN
               RAISE ERR_PARTIT_NEXIST;
         END;
         
         IF v_databaixa IS NULL 
         THEN
            RAISE ERR_PARTIT_OPERATIU;
         ELSE
            UPDATE partit 
               SET datamod = sysdate,
                   usumod = p_usuari,
                   databaixa = NULL,
                   usubaixa = NULL
             WHERE id = p_id;
         END IF;
         
         COMMIT;
         RSP := 'OK';
      EXCEPTION
         WHEN ERR_PARAMS_NULL
         THEN
            RSP := PK_ERRORS.ERR_PARAMS_NULL;
         WHEN ERR_USER
         THEN
            RSP := PK_ERRORS.ERR_USER;
         WHEN ERR_PARTIT_NEXIST
         THEN
            RSP := PK_ERRORS.ERR_PARTIT_NEXIST;
         WHEN ERR_PARTIT_OPERATIU
         THEN
            RSP := PK_ERRORS.ERR_PARTIT_OPERATIU;
         WHEN OTHERS THEN
			  RSP := CONCAT(PK_ERRORS.ERR_NOTRACTAT, (TO_CHAR(SQLCODE) || ' '|| SUBSTR(SQLERRM,1,400)));
			  ROLLBACK;
		END;
      p_log (v_nomproc, v_parin, RSP);
   END react_partit;
   
	PROCEDURE alta_parlamentari (
		p_document		IN	   parlamentari.document%TYPE,
		p_parlament 	IN	   parlamentari.parlament%TYPE,
		p_partit		   IN 	parlamentari.partit%TYPE,
		p_nom       	IN	   parlamentari.nom%TYPE,
		p_cognoms   	IN	   parlamentari.cognoms%TYPE,
		p_numIdentificacio 	IN	parlamentari.numidentificacio%TYPE,
		p_numTargeta	IN	   parlamentari.numtargeta%TYPE,
		p_usuari		   IN 	parlamentari.usualta%TYPE,
		RSP				OUT	NOCOPY VARCHAR2
	) IS
		v_nomproc	         logs.nomproc%TYPE := 'PK_ABM.alta_parlamentari';
		v_parin		         logs.parin%TYPE;
		
      v_exist              NUMBER;

      
      ERR_PARAMS_NULL         EXCEPTION;
      ERR_USER                EXCEPTION;
      ERR_BAIXA_PARLAMENT     EXCEPTION;
      ERR_BAIXA_PARTIT        EXCEPTION;
      ERR_DUP_PARL            EXCEPTION;
      ERR_DUP_IDENTPARL       EXCEPTION;
	BEGIN
		v_parin := CONCAT('p_document=', p_document);
		v_parin := v_parin || CONCAT('; p_parlament=', p_parlament);
		v_parin := v_parin || CONCAT('; p_partit=', p_partit);
		v_parin := v_parin || CONCAT('; p_nom=', p_nom);
		v_parin := v_parin || CONCAT('; p_cognoms=', p_cognoms);
		v_parin := v_parin || CONCAT('; p_numIdentificacio=', p_numIdentificacio);
		v_parin := v_parin || CONCAT('; p_numTargeta=', p_numTargeta);
		v_parin := v_parin || CONCAT('; p_usuari=', p_usuari);
      
		BEGIN
         -- Validació dels paràmetres d'entrada
         IF p_document IS NULL OR p_parlament IS NULL OR p_partit IS NULL
            OR p_nom IS NULL OR p_cognoms IS NULL OR p_numidentificacio IS NULL
         THEN
            RAISE ERR_PARAMS_NULL;
         END IF;
         
         IF p_usuari IS NULL
         THEN
            RAISE ERR_USER;
         END IF;
         
         -- Comprova que el parlament està operatiu
         IF NOT isParlamentOperatiu (p_parlament)
         THEN            
            RAISE ERR_BAIXA_PARLAMENT;
         END IF;         
         
         -- Comprova que el partit està operatiu
         IF NOT isPartitOperatiu (p_partit)
         THEN
            RAISE ERR_BAIXA_PARTIT;
         END IF;
         
         -- Comprova que no existeixi un altre parlamentari amb el mateix document
         SELECT COUNT(1) INTO v_exist
           FROM parlamentari
          WHERE document = UPPER(p_document);
          
         IF v_exist != 0 
         THEN
            RAISE ERR_DUP_PARL;
         END IF;
         
         -- Comprova que no existeixi un altre parlamentari dins el mateix parlament
         -- amb el mateix número identificatiu
         SELECT COUNT(1) INTO v_exist
           FROM parlamentari
          WHERE numidentificacio = UPPER(p_numidentificacio)
            AND parlament = p_parlament;
         
         IF v_exist != 0
         THEN
            RAISE ERR_DUP_IDENTPARL;
         END IF;
         
		
			INSERT INTO parlamentari (id, document, parlament, partit, nom,
						cognoms, numidentificacio, numtargeta, usualta, 
						dataalta)
				VALUES (seq_parlamentari.NEXTVAL, UPPER(p_document), p_parlament, p_partit, p_nom,
						p_cognoms, UPPER(p_numidentificacio), p_numtargeta, p_usuari,
						sysdate);
			COMMIT;
			RSP :=  'OK';
		EXCEPTION
         WHEN ERR_PARAMS_NULL
         THEN
            RSP := PK_ERRORS.ERR_PARAMS_NULL;
         WHEN ERR_USER
         THEN
            RSP := PK_ERRORS.ERR_USER;
         WHEN ERR_BAIXA_PARLAMENT
         THEN
            RSP := PK_ERRORS.ERR_BAIXA_PARLAMENT;
         WHEN ERR_BAIXA_PARTIT
         THEN
            RSP := PK_ERRORS.ERR_BAIXA_PARTIT;
         WHEN ERR_DUP_PARL
         THEN
            RSP := PK_ERRORS.ERR_DUP_PARL;
         WHEN ERR_DUP_IDENTPARL
         THEN
            RSP := PK_ERRORS.ERR_DUP_IDENTPARL;
			WHEN OTHERS THEN
			  RSP := CONCAT(PK_ERRORS.ERR_NOTRACTAT, (TO_CHAR(SQLCODE) || ' '|| SUBSTR(SQLERRM,1,400)));
			  ROLLBACK;
		END;
		p_log (v_nomproc, v_parin, RSP);
	END alta_parlamentari;
	
	PROCEDURE modif_parlamentari (
		p_id			   IN	   parlamentari.id%TYPE,
		p_document		IN	   parlamentari.document%TYPE,
		p_parlament 	IN	   parlamentari.parlament%TYPE,
		p_partit		   IN 	parlamentari.partit%TYPE,
		p_nom       	IN	   parlamentari.nom%TYPE,
		p_cognoms   	IN	   parlamentari.cognoms%TYPE,
		p_numIdentificacio 	IN	parlamentari.numidentificacio%TYPE,
		p_numTargeta	IN	   parlamentari.numtargeta%TYPE,
		p_usuari		   IN 	parlamentari.usumod%TYPE,
		RSP				OUT	NOCOPY VARCHAR2
	)  IS
		v_nomproc	logs.nomproc%TYPE := 'PK_ABM.modif_parlamentari';
		v_parin		logs.parin%TYPE;
      
      v_exist       NUMBER;
      
      ERR_PARAMS_NULL         EXCEPTION;
      ERR_USER                EXCEPTION;
      ERR_BAIXA_PARLAMENT     EXCEPTION;
      ERR_BAIXA_PARTIT        EXCEPTION;
      ERR_DUP_PARL            EXCEPTION;
      ERR_DUP_IDENTPARL       EXCEPTION;
      
      
	BEGIN
		v_parin := CONCAT('p_id=', CONCAT(p_id,'; '));
		v_parin := v_parin || CONCAT('; p_document=', p_document);
		v_parin := v_parin || CONCAT('; p_parlament=', p_parlament);
		v_parin := v_parin || CONCAT('; p_partit=', p_partit);
		v_parin := v_parin || CONCAT('; p_nom=', p_nom);
		v_parin := v_parin || CONCAT('; p_cognoms=', p_cognoms);
		v_parin := v_parin || CONCAT('; p_numIdentificacio=', p_numIdentificacio);
		v_parin := v_parin || CONCAT('; p_numTargeta=', p_numTargeta);
		v_parin := v_parin || CONCAT('; p_usuari=', p_usuari);
      
		BEGIN
         -- Validació dels paràmetres d'entrada
         IF p_id IS NULL OR p_document IS NULL OR p_parlament IS NULL
            OR p_nom IS NULL OR p_cognoms IS NULL OR p_numidentificacio IS NULL
         THEN
            RAISE ERR_PARAMS_NULL;
         END IF;
         
         IF p_usuari IS NULL
         THEN
            RAISE ERR_USER;
         END IF;
         
         -- Mira que el parlament estigui operatiu
         IF NOT isParlamentOperatiu(p_parlament)
         THEN
            RAISE ERR_BAIXA_PARLAMENT;
         END IF;
         
         -- Mira que el partit estigui operatiu
         IF NOT isPartitOperatiu(p_parlament)
         THEN
            RAISE ERR_BAIXA_PARTIT;
         END IF;
         
         -- Valida que no hi hagi un altre parlamentari amb el mateix document
         SELECT COUNT(1) INTO v_exist
           FROM parlamentari
          WHERE document = UPPER(p_document)
            AND id != p_id;
            
         IF v_exist != 0 
         THEN
            RAISE ERR_DUP_PARL;
         END IF;
         
         -- Valida que número identificatiu sigui únic dins el parlament
         SELECT COUNT(1) INTO v_exist
           FROM parlamentari
          WHERE numIdentificacio = UPPER(p_numIdentificacio)
            AND parlament = p_parlament
            AND id != p_id;
         
         IF v_exist != 0
         THEN
            RAISE ERR_DUP_IDENTPARL;
         END IF;
         
			UPDATE parlamentari
			   SET document = UPPER(p_document),
				   parlament = p_parlament,
				   partit = p_partit,
				   nom = p_nom,
				   cognoms = p_cognoms,
				   numidentificacio = UPPER(p_numidentificacio),
				   numtargeta = p_numtargeta,
				   usumod = p_usuari,
				   datamod = sysdate
			 WHERE id = p_id;
			COMMIT;
			RSP :=  'OK';
		EXCEPTION
         WHEN ERR_PARAMS_NULL
         THEN
            RSP := PK_ERRORS.ERR_PARAMS_NULL;
         WHEN ERR_USER
         THEN
            RSP := PK_ERRORS.ERR_USER;
         WHEN ERR_BAIXA_PARLAMENT
         THEN
            RSP := PK_ERRORS.ERR_BAIXA_PARLAMENT;
         WHEN ERR_DUP_PARL
         THEN
            RSP := PK_ERRORS.ERR_DUP_PARL;
         WHEN ERR_DUP_IDENTPARL
         THEN
            RSP := PK_ERRORS.ERR_DUP_IDENTPARL;
			WHEN OTHERS THEN
			  RSP := CONCAT(PK_ERRORS.ERR_NOTRACTAT, (TO_CHAR(SQLCODE) || ' '|| SUBSTR(SQLERRM,1,400)));
			  ROLLBACK;
		END;
		p_log (v_nomproc, v_parin, RSP);
	END modif_parlamentari;
	
	
	PROCEDURE baixa_parlamentari (
		p_id		IN		parlamentari.id%TYPE,
		p_usuari	IN		parlamentari.usubaixa%TYPE,
		RSP			OUT		NOCOPY VARCHAR2
	)  IS
		v_nomproc	logs.nomproc%TYPE := 'PK_ABM.baixa_parlamentari';
		v_parin		logs.parin%TYPE;
      
      ERR_PARAMS_NULL      EXCEPTION;
      ERR_USER             EXCEPTION;
      ERR_BAIXA_PARL       EXCEPTION;
	BEGIN
		v_parin := CONCAT('p_id=', p_id);
		v_parin := v_parin ||  CONCAT('; p_usuari=', p_usuari);
		BEGIN
         -- Validació dels paràmetres d'entrada
         IF p_id IS NULL
         THEN
            RAISE ERR_PARAMS_NULL;
         END IF;
         
         IF p_usuari IS NULL
         THEN
            RAISE ERR_USER;
         END IF;
         
         -- Mira que el parlamentari estigui operatiu
         IF NOT isParlamentariOperatiu (p_id)
         THEN
            RAISE ERR_BAIXA_PARL;
         END IF;
         
			UPDATE parlamentari
			   SET usubaixa = p_usuari,
			       databaixa = sysdate
			 WHERE id = p_id;
			COMMIT;
			RSP :=  'OK';
		EXCEPTION
         WHEN ERR_PARAMS_NULL
         THEN
            RSP := PK_ERRORS.ERR_PARAMS_NULL;
         WHEN ERR_USER 
         THEN
            RSP := PK_ERRORS.ERR_USER;
         WHEN ERR_BAIXA_PARL
         THEN
            RSP := PK_ERRORS.ERR_BAIXA_PARL;
			WHEN OTHERS THEN
			  RSP := CONCAT(PK_ERRORS.ERR_NOTRACTAT, (TO_CHAR(SQLCODE) || ' '|| SUBSTR(SQLERRM,1,400)));
			  ROLLBACK;
		END;
		p_log (v_nomproc, v_parin, RSP);
	END baixa_parlamentari;
	
   PROCEDURE react_parlamentari (
      p_id           IN    parlamentari.id%TYPE,
      p_parlament    IN    parlamentari.parlament%TYPE,
      p_partit       IN    parlamentari.partit%TYPE,      
      p_usuari       IN    parlamentari.usumod%TYPE,
      RSP            OUT   NOCOPY VARCHAR2
   )
   IS
      v_nomproc	            logs.nomproc%TYPE := 'PK_ABM.react_parlamentari';
		v_parin		            logs.parin%TYPE;
      
      v_databaixa             parlamentari.databaixa%TYPE;
     
      ERR_PARAMS_NULL         EXCEPTION;
      ERR_USER                EXCEPTION;
      ERR_PARLA_OPERATIU      EXCEPTION;
      ERR_PARLA_NEXIST        EXCEPTION;
      ERR_PARLAMENT_NOPERATIU EXCEPTION;
      ERR_PARTIT_NOPERATIU    EXCEPTION;
   BEGIN
      v_parin := CONCAT ('p_id=', p_id);
      v_parin := v_parin || CONCAT('; p_parlament=', p_parlament);
      v_parin := v_parin || CONCAT('; p_partit=', p_partit);
      v_parin := v_parin || CONCAT('; p_usuari=', p_usuari);
      
      BEGIN
         -- Validació dels paràmetres d'entrada
         IF p_id IS NULL OR p_parlament IS NULL OR p_partit IS NULL
         THEN
            RAISE ERR_PARAMS_NULL;
         END IF;
         
         IF p_usuari IS NULL
         THEN
            RAISE ERR_USER;
         END IF;
         
         -- Valida que el parlamentari existeixi i que estigui donat de baixa
         BEGIN
            SELECT databaixa
              INTO v_databaixa
              FROM parlamentari
             WHERE id = p_id;
         EXCEPTION 
            WHEN NO_DATA_FOUND
            THEN
               RAISE ERR_PARLA_NEXIST;
         END;
         
         IF v_databaixa IS NULL 
         THEN
            RAISE ERR_PARLA_OPERATIU;
         END IF;
         
         -- Comprova que el parlament estigui operatiu
         IF NOT isParlamentOperatiu (p_parlament)
         THEN
            RAISE ERR_PARLAMENT_NOPERATIU;
         END IF;
         
         -- Comprova que el partit està operatiu
         IF NOT isPartitOperatiu (p_partit)
         THEN
            RAISE ERR_PARTIT_NOPERATIU;
         END IF;
         

         UPDATE parlamentari 
            SET datamod = sysdate,
                usumod = p_usuari,
                databaixa = NULL,
                usubaixa = NULL
          WHERE id = p_id;
         
         COMMIT;
         RSP := 'OK';
      EXCEPTION
         WHEN ERR_PARAMS_NULL
         THEN
            RSP := PK_ERRORS.ERR_PARAMS_NULL;
         WHEN ERR_USER
         THEN
            RSP := PK_ERRORS.ERR_USER;
         WHEN ERR_PARTIT_NOPERATIU
         THEN
            RSP := PK_ERRORS.ERR_PARTIT_NOPERATIU;
         WHEN ERR_PARLAMENT_NOPERATIU
         THEN
            RSP := PK_ERRORS.ERR_PARLAMENT_NOPERATIU;
         WHEN ERR_PARLA_NEXIST
         THEN
            RSP := PK_ERRORS.ERR_PARLA_NEXIST;
         WHEN ERR_PARLA_OPERATIU
         THEN
            RSP := PK_ERRORS.ERR_PARLA_OPERATIU;
         WHEN OTHERS THEN
			  RSP := CONCAT(PK_ERRORS.ERR_NOTRACTAT, (TO_CHAR(SQLCODE) || ' '|| SUBSTR(SQLERRM,1,400)));
			  ROLLBACK;
		END;
      p_log (v_nomproc, v_parin, RSP);
   END react_parlamentari;
   
	PROCEDURE alta_despesa (
      p_origendespesa IN   despesa.origendespesa%TYPE,
		p_codidespesa	IN	   despesa.codidespesa%TYPE,
		p_parlament    IN	   despesa.parlament%TYPE,
		p_parlamentari	IN	   despesa.parlamentari%TYPE,
		p_partit		   IN	   despesa.partit%TYPE,
		p_data         IN	   despesa.data%TYPE,
		p_ibrut        IN	   despesa.ibrut%TYPE,
		p_iva          IN	   despesa.iva%TYPE,
		p_descripcio   IN	   despesa.descripcio%TYPE,
		p_justificacio IN	   despesa.justificacio%TYPE,
		p_urldocument  IN	   despesa.urldocument%TYPE,
		p_formapagament IN	despesa.formapagament%TYPE,
		p_estatdespesa IN	   despesa.estatdespesa%TYPE,
      p_iban         IN    despesa.iban%TYPE,
      p_numtargeta   IN    despesa.numtargeta%TYPE,
		p_empresa	   IN	   despesa.empresa%TYPE,
		p_usuari		   IN	   despesa.usualta%TYPE,
      RSP            OUT   NOCOPY VARCHAR2
	)
	IS
      v_nomproc	            logs.nomproc%TYPE := 'PK_ABM.alta_despesa';
		v_parin		            logs.parin%TYPE;
      
      v_inet                  despesa.inet%TYPE;
      v_anyo                  despesa.anyo%TYPE;
	BEGIN       
      v_parin := CONCAT('p_origendespesa=', p_origendespesa);
      v_parin := v_parin || CONCAT('; p_codidespesa=', p_codidespesa);
      v_parin := v_parin || CONCAT('; p_parlament=', p_parlament);
      v_parin := v_parin || CONCAT('; p_parlamentari=', p_parlamentari);
      v_parin := v_parin || CONCAT('; p_partit=', p_partit);
      v_parin := v_parin || CONCAT('; p_data=', p_data);
      v_parin := v_parin || CONCAT('; p_ibrut=', p_ibrut);
      v_parin := v_parin || CONCAT('; p_iva=', p_iva);
      v_parin := v_parin || CONCAT('; p_descripcio=', p_descripcio);
      v_parin := v_parin || CONCAT('; p_justificacio=', p_justificacio);
      v_parin := v_parin || CONCAT('; p_urldocument=', p_urldocument);
      v_parin := v_parin || CONCAT('; p_formapagament=', p_formapagament);
      v_parin := v_parin || CONCAT('; p_estatdespesa=', p_estatdespesa);
      v_parin := v_parin || CONCAT('; p_iban=', p_iban);
      v_parin := v_parin || CONCAT('; p_numtargeta=', p_numtargeta);
      v_parin := v_parin || CONCAT('; p_empresa=', p_empresa);
      v_parin := v_parin || CONCAT('; p_usuari=', p_usuari);
      
      -- Aquesta funció no fa cap valiació perquè és un procediment intern, la validació
      -- de dades ja la fan els procediments abans de cridar a aquest.
      
		INSERT INTO despesa (id, origendespesa, codidespesa,                      
                           parlament, parlamentari, partit, data,
                           ibrut, iva, descripcio, 
                           justificacio, urldocument, formapagament, 
                           estatdespesa, iban, numtargeta, empresa, usualta, dataalta)
                   VALUES (seq_despesa.NEXTVAL, p_origendespesa, p_codidespesa, 
                           p_parlament, p_parlamentari, p_partit, p_data, 
                           p_ibrut, p_iva, p_descripcio, 
                           p_justificacio, p_urldocument, p_formapagament, 
                           p_estatdespesa, p_iban, p_numtargeta, p_empresa, p_usuari, sysdate)
                 RETURNING inet, anyo  INTO v_inet, v_anyo;      
      
      -- Crida al procediment d'estadística per enregistrar la nova despesa dins
      -- el mòdul estadístic.
      PK_ESTADISTICA.alta_despesa ( p_any             => v_anyo,
                                    p_inet            => v_inet,
                                    p_parlament       => p_parlament,
                                    p_parlamentari    => p_parlamentari,
                                    p_origenDespesa   => p_origenDespesa,
                                    p_estatDespesa    => p_estatDespesa,
                                    RSP               => RSP
                                  );
      RSP := 'OK';
      p_log (v_nomproc, v_parin, RSP);                                  
   EXCEPTION
      WHEN OTHERS 
      THEN
         -- En el cas que es produesqui qualsevol error el registre i propaga l'error.
         p_log (v_nomproc, v_parin, RSP);
         RAISE;
	END alta_despesa;
	
	PROCEDURE modif_despesa (
		p_id			   IN 	despesa.id%TYPE,
		p_codidespesa	IN 	despesa.codidespesa%TYPE,
		p_data         IN	   despesa.data%TYPE,
		p_ibrut        IN	   despesa.ibrut%TYPE,
		p_iva          IN	   despesa.iva%TYPE,
		p_descripcio   IN	   despesa.descripcio%TYPE,
		p_justificacio IN	   despesa.justificacio%TYPE,
		p_urldocument  IN	   despesa.urldocument%TYPE,
		p_formapagament IN	despesa.formapagament%TYPE,
		p_estatdespesa IN	   despesa.estatdespesa%TYPE,
      p_iban         IN    despesa.iban%TYPE,
      p_numtargeta   IN    despesa.numtargeta%TYPE,
		p_empresa		IN	despesa.empresa%TYPE,
		p_usuari		   IN	   despesa.usualta%TYPE,
      RSP            OUT   NOCOPY VARCHAR2
	)
	IS		
      v_nomproc	            logs.nomproc%TYPE := 'PK_ABM.modif_despesa';
		v_parin		            logs.parin%TYPE;
      
      v_anyOld                despesa.anyo%TYPE;
      v_inetOld               despesa.inet%TYPE;
      v_estatDespesaOld       despesa.estatDespesa%TYPE;
      v_anyNew                despesa.anyo%TYPE;
      v_inetNew               despesa.inet%TYPE;
      v_origenDespesa         despesa.origenDespesa%TYPE;
      v_parlament             despesa.parlament%TYPE;
      v_parlamentari          despesa.parlamentari%TYPE;
	BEGIN
      v_parin := CONCAT('p_id=', p_id);
      v_parin := v_parin || CONCAT('; p_codidespesa=', p_codidespesa);
      v_parin := v_parin || CONCAT('; p_data=', p_data);
      v_parin := v_parin || CONCAT('; p_ibrut=', p_ibrut);
      v_parin := v_parin || CONCAT('; p_iva=', p_iva);
      v_parin := v_parin || CONCAT('; p_descripcio=', p_descripcio);
      v_parin := v_parin || CONCAT('; p_justificacio=', p_justificacio);
      v_parin := v_parin || CONCAT('; p_urldocument=', p_urldocument);
      v_parin := v_parin || CONCAT('; p_formapagament=', p_formapagament);
      v_parin := v_parin || CONCAT('; p_estatdespesa=', p_estatdespesa);
      v_parin := v_parin || CONCAT('; p_iban=', p_iban);
      v_parin := v_parin || CONCAT('; p_numtargeta=', p_numtargeta);
      v_parin := v_parin || CONCAT('; p_empresa=', p_empresa);
      v_parin := v_parin || CONCAT('; p_usuari=', p_usuari);
      
      -- Aquest procedument no fa cap validació dels paràmetres d'entrada perquè és
      -- un procediment intern, les valiacions les fan el procediments abans de cridar
      -- a aquest.
      
      -- Recuperam els valors necessaris pel mòdul estadístic, segons els tipus de modificació
      -- que és faci és necessari desenregistrar el valors per després enregistrar els nous.
   
      SELECT anyo, inet, origenDespesa, parlament, parlamentari, estatDespesa 
        INTO v_anyOld, v_inetOld, v_origenDespesa, v_parlament, v_parlamentari, v_estatDespesaOld
        FROM despesa 
       WHERE id = p_id;
       
		UPDATE despesa
		   SET codidespesa = p_codidespesa,
			   data = p_data,
			   ibrut = p_ibrut,
			   iva = p_iva,
			   descripcio = p_descripcio,
			   justificacio = p_justificacio,
			   urldocument = p_urldocument,
			   formapagament = p_formapagament,
			   estatdespesa = p_estatdespesa,
            iban = p_iban,
            numtargeta = p_numtargeta,
				empresa = p_empresa,
			   usumod = p_usuari,
			   datamod = sysdate
		 WHERE id = p_id
       RETURNING anyo, inet
            INTO v_anyNew, v_inetNew;		 
      
      
      -- Crida al mòdul estadśitic per enregistrar la modificació
      PK_ESTADISTICA.modif_despesa ( p_anyOld            => v_anyOld,
                                     p_inetOld           => v_inetOld,
                                     p_estatDespesaOld   => v_estatDespesaOld,
                                     p_anyNew            => v_anyNew,
                                     p_inetNew           => v_inetNew,
                                     p_estatDespesaNew   => p_estatDespesa,
                                     p_parlament         => v_parlament,
                                     p_parlamentari      => v_parlamentari,
                                     p_origenDespesa     => v_origenDespesa,
                                     RSP                 => RSP
                                   );         
      RSP := 'OK';
      p_log (v_nomproc, v_parin, RSP);
   EXCEPTION
      WHEN OTHERS
      THEN
         -- En cas de qualsevol error, enregistra l'error i propaga l'excepció
         p_log (v_nomproc, v_parin, RSP);
         RAISE;
	END modif_despesa;
	
	PROCEDURE alta_despesa_parlament (
		p_codidespesa  IN	   despesa.codidespesa%TYPE,
		p_parlament    IN	   despesa.parlament%TYPE,
		p_data        IN	   despesa.data%TYPE,
		p_ibrut        IN	   despesa.ibrut%TYPE,
		p_iva          IN	   despesa.iva%TYPE,
		p_descripcio   IN	   despesa.descripcio%TYPE,
		p_justificacio IN	   despesa.justificacio%TYPE,
		p_urldocument  IN	   despesa.urldocument%TYPE,
		p_formapagament IN	despesa.formapagament%TYPE,
		p_estatdespesa IN	   despesa.estatdespesa%TYPE,
      p_iban         IN    despesa.iban%TYPE,
      p_numtargeta   IN    despesa.numtargeta%TYPE,
		p_usuari		   IN	   despesa.usualta%TYPE,
		RSP				OUT	NOCOPY VARCHAR2
	)  IS
		v_nomproc	logs.nomproc%TYPE := 'PK_ABM.alta_despesa_parlament';
		v_parin		logs.parin%TYPE;
      
      v_count           NUMBER;
      
      ERR_PARAMS_NULL      EXCEPTION;
      ERR_USER             EXCEPTION;
      ERR_BAIXA_PARLAMENT  EXCEPTION;
      ERR_CODI_DESPESA     EXCEPTION;
      ERR_ESTAT_DESPESA    EXCEPTION;
      ERR_FORMA_PAGAMENT   EXCEPTION;
      ERR_IBAN             EXCEPTION;
      ERR_CREDITCARD       EXCEPTION;
      ERR_IBAN_CREDITCARD  EXCEPTION;
      ERR_IBRUT            EXCEPTION;
      ERR_IVA              EXCEPTION;
	BEGIN
		v_parin := CONCAT('p_codidespesa=', p_codidespesa);
		v_parin := v_parin || CONCAT(' p_parlament=', p_parlament);
		v_parin := v_parin || CONCAT(' p_data=', p_data);
		v_parin := v_parin || CONCAT(' p_ibrut=', p_ibrut);
		v_parin := v_parin || CONCAT(' p_iva=', p_iva);
		v_parin := v_parin || CONCAT(' p_descripcio=', p_descripcio);
		v_parin := v_parin || CONCAT(' p_justificacio=', p_justificacio);
		v_parin := v_parin || CONCAT(' p_urldocument=', p_urldocument);
		v_parin := v_parin || CONCAT(' p_formapagament=', p_formapagament);
		v_parin := v_parin || CONCAT(' p_estatdespesa=', p_estatdespesa);
      v_parin := v_parin || CONCAT(' p_iban=', p_iban);
      v_parin := v_parin || CONCAT(' p_numtargeta=', p_numtargeta);
		v_parin := v_parin || CONCAT(' p_usuari=', p_usuari);
      
		BEGIN
         -- Validació dels paràmetres d'entrada
         IF p_codidespesa IS NULL OR p_parlament IS NULL OR p_data IS NULL
            OR p_ibrut IS NULL OR p_iva IS NULL OR p_descripcio IS NULL
            OR p_justificacio IS NULL OR p_formapagament IS NULL
            OR p_estatdespesa IS NULL 
         THEN
            RAISE ERR_PARAMS_NULL;
         END IF;
         
         IF p_usuari IS NULL
         THEN
            RAISE ERR_USER;
         END IF;
         
         -- Mira que el parlament estigui operatiu
         IF NOT isParlamentOperatiu(p_parlament)
         THEN
            RAISE ERR_BAIXA_PARLAMENT;
         END IF;
         
         -- Mira que el codi de despesa sigui correcta i que aquest pertanyi al
         -- tipus de despesa de parlament
         SELECT COUNT(1) INTO v_count
           FROM tipusdespesaparlament
          WHERE codi = p_codidespesa;
          
         IF v_count = 0 
         THEN
            RAISE ERR_CODI_DESPESA;            
         END IF;
         
         -- Comprova el codi d'estat de despesa
         SELECT COUNT(1) INTO v_count
           FROM estatdespesa
          WHERE id = p_estatdespesa;
          
         IF v_count = 0
         THEN
            RAISE ERR_ESTAT_DESPESA;
         END IF;
         
         -- Comprova la codificació de la forma de pagament
         SELECT count(1) INTO v_count
           FROM formapagament
          WHERE id = p_formapagament;
          
         IF v_count = 0 
         THEN
            RAISE ERR_FORMA_PAGAMENT;
         END IF;         
         
         -- En el cas que la forma de pagament sigui per compte és necessari
         -- informar el IBAN, si la forma de pagament és targeta és necessari
         -- informar el número de targeta.
         -- En cap cas es pot informar el número de compte i el número de targeta
         IF p_formapagament = 1 AND p_iban IS NULL
         THEN
            RAISE ERR_IBAN;
         ELSIF p_formapagament = 2 AND p_numtargeta IS NULL
         THEN
            RAISE ERR_CREDITCARD;
         ELSIF p_iban IS NOT NULL AND p_numtargeta IS NOT NULL
         THEN
            RAISE ERR_IBAN_CREDITCARD;
         END IF;

         -- Valida que l'import brut sigui superior a 0.
         IF p_ibrut <= 0 
         THEN
            RAISE ERR_IBRUT;
         END IF;
         
         -- Valida que l'iva no sigui negatiu
         IF p_iva < 0
         THEN
            RAISE ERR_IVA;
         END IF;                  

         -- Crida al procediment intern que enregistrarà la despesa.
			alta_despesa (
            p_origendespesa   => 1,
            p_codidespesa     => p_codidespesa,
            p_parlament     	=> p_parlament,
            p_parlamentari	   => NULL,
            p_partit			   => NULL,
            p_data         	=> p_data,
				p_ibrut         	=> p_ibrut,
				p_iva           	=> p_iva,
				p_descripcio    	=> p_descripcio,
				p_justificacio  	=> p_justificacio,
				p_urldocument   	=> p_urldocument,
				p_formapagament 	=> p_formapagament,
				p_estatdespesa  	=> p_estatdespesa,
            p_iban            => p_iban,
            p_numtargeta      => p_numtargeta,
            p_empresa         => NULL,
				p_usuari        	=> p_usuari,
            RSP               => RSP);

			COMMIT;
			RSP :=  'OK';
		EXCEPTION
         WHEN ERR_PARAMS_NULL
         THEN
            RSP := PK_ERRORS.ERR_PARAMS_NULL;
         WHEN ERR_USER             
         THEN
            RSP := PK_ERRORS.ERR_USER;
         WHEN ERR_BAIXA_PARLAMENT  
         THEN
            RSP := PK_ERRORS.ERR_BAIXA_PARLAMENT;
         WHEN ERR_CODI_DESPESA
         THEN
            RSP := PK_ERRORS.ERR_CODI_DESPESA;
         WHEN ERR_ESTAT_DESPESA
         THEN
            RSP := PK_ERRORS.ERR_ESTAT_DESPESA;
         WHEN ERR_FORMA_PAGAMENT   
         THEN
            RSP := PK_ERRORS.ERR_FORMA_PAGAMENT;
         WHEN ERR_IBAN       
         THEN
            RSP := PK_ERRORS.ERR_IBAN;
         WHEN ERR_CREDITCARD 
         THEN
            RSP := PK_ERRORS.ERR_CREDITCARD;
         WHEN ERR_IBAN_CREDITCARD  
         THEN
            RSP := PK_ERRORS.ERR_IBAN_CREDITCARD;
         WHEN ERR_IBRUT      
         THEN
            RSP := PK_ERRORS.ERR_IBRUT;
         WHEN ERR_IVA        
         THEN
            RSP := PK_ERRORS.ERR_IVA;
			WHEN OTHERS 
         THEN
            IF RSP IS NULL
            THEN
               RSP := CONCAT(PK_ERRORS.ERR_NOTRACTAT, (TO_CHAR(SQLCODE) || ' '|| SUBSTR(SQLERRM,1,400)));
            END IF;
            ROLLBACK;
		END;
		p_log (v_nomproc, v_parin, RSP);
	END alta_despesa_parlament;
	
	PROCEDURE modif_despesa_parlament (
		p_id			   IN	   despesa.id%TYPE,
		p_codidespesa	IN	   despesa.codidespesa%TYPE,
		p_data        IN	   despesa.data%TYPE,
		p_ibrut        IN	   despesa.ibrut%TYPE,
		p_iva          IN	   despesa.iva%TYPE,
		p_descripcio   IN	   despesa.descripcio%TYPE,
		p_justificacio IN	   despesa.justificacio%TYPE,
		p_urldocument  IN	   despesa.urldocument%TYPE,
		p_formapagament IN	despesa.formapagament%TYPE,
		p_estatdespesa IN    despesa.estatdespesa%TYPE,
      p_iban         IN    despesa.iban%TYPE,
      p_numtargeta   IN    despesa.numtargeta%TYPE,
		p_usuari		   IN	   despesa.usumod%TYPE,
		RSP				OUT	NOCOPY VARCHAR2
	)  
   IS
		v_nomproc	logs.nomproc%TYPE := 'PK_ABM.modif_despesa_parlament';
		v_parin		logs.parin%TYPE;
      
      v_exist                 NUMBER;
      
      ERR_PARAMS_NULL         EXCEPTION;
      ERR_USER                EXCEPTION;
      ERR_CODI_DESPESA        EXCEPTION;
      ERR_ESTAT_DESPESA       EXCEPTION;
      ERR_FORMA_PAGAMENT      EXCEPTION;
      ERR_IBAN                EXCEPTION;
      ERR_CREDITCARD          EXCEPTION;
      ERR_IBAN_CREDITCARD     EXCEPTION;
      ERR_IBRUT               EXCEPTION;
      ERR_IVA                 EXCEPTION;
	BEGIN
		v_parin := CONCAT('p_id=', p_id);
		v_parin := v_parin || CONCAT('; p_codidespesa=', p_codidespesa);
		v_parin := v_parin || CONCAT('; p_data=', p_data);
		v_parin := v_parin || CONCAT('; p_ibrut=', p_ibrut);
		v_parin := v_parin || CONCAT('; p_iva=', p_iva);
		v_parin := v_parin || CONCAT('; p_descripcio=', p_descripcio);
		v_parin := v_parin || CONCAT('; p_justificacio=', p_justificacio);
		v_parin := v_parin || CONCAT('; p_urldocument=', p_urldocument);
		v_parin := v_parin || CONCAT('; p_formapagament=', p_formapagament);
		v_parin := v_parin || CONCAT('; p_estatdespesa=', p_estatdespesa);
      v_parin := v_parin || CONCAT('; p_iban=', p_iban);
      v_parin := v_parin || CONCAT('; p_numtargeta=', p_numtargeta);
		v_parin := v_parin || CONCAT('; p_usuari=', p_usuari);

		BEGIN
         -- Validació dels paràmetres d'entrada
         IF p_id IS NULL OR p_codidespesa IS NULL OR p_data IS NULL
            OR p_ibrut IS NULL OR p_iva IS NULL OR p_descripcio IS NULL
            OR p_justificacio IS NULL OR p_urldocument IS NULL
            OR p_formapagament IS NULL OR p_estatdespesa IS NULL
         THEN
            RAISE ERR_PARAMS_NULL;
         END IF;
         
         IF p_usuari IS NULL
         THEN
            RAISE ERR_USER;
         END IF;
         
         -- Valida el codi de la despesa
         SELECT COUNT(1) INTO v_exist
           FROM tipusdespesaparlament
          WHERE codi = p_codidespesa;
          
         IF v_exist = 0 
         THEN
            RAISE ERR_CODI_DESPESA;
         END IF;
         
         -- Valida l'estat de la despesa
         SELECT COUNT(1) INTO v_exist
           FROM estatdespesa
          WHERE id = p_estatdespesa;
          
         IF v_exist = 0
         THEN
            RAISE ERR_ESTAT_DESPESA;
         END IF;
         
         -- Valida la forma de pagament
         SELECT count(1) INTO v_exist
           FROM formapagament
          WHERE id = p_formapagament;
          
         IF v_exist = 0 
         THEN
            RAISE ERR_FORMA_PAGAMENT;
         END IF;         
         
         -- En el cas que la forma de pagament sigui per compte és necessari
         -- informar el IBAN, si la forma de pagament és targeta és necessari
         -- informar el número de targeta.
         -- En cap cas es pot informar el número de compte i el número de targeta
         IF p_formapagament = 1 AND p_iban IS NULL
         THEN
            RAISE ERR_IBAN;
         ELSIF p_formapagament = 2 AND p_numtargeta IS NULL
         THEN
            RAISE ERR_CREDITCARD;
         ELSIF p_iban IS NOT NULL AND p_numtargeta IS NOT NULL
         THEN
            RAISE ERR_IBAN_CREDITCARD;
         END IF;
         
         -- Comprova que l'import és correcta
         IF p_ibrut <= 0 
         THEN
            RAISE ERR_IBRUT;
         END IF;
         
         -- Comprova que l'iva no és negatiu
         IF p_iva < 0
         THEN
            RAISE ERR_IVA;
         END IF;         
         
         -- Crida al procediment intern per modificar la despesa
			modif_despesa (p_id				=> p_id,
						   p_codidespesa	=> p_codidespesa,
						   p_data         	=> p_data,
						   p_ibrut         	=> p_ibrut,
						   p_iva           	=> p_iva,
						   p_descripcio    	=> p_descripcio,
						   p_justificacio  	=> p_justificacio,
						   p_urldocument   	=> p_urldocument,
						   p_formapagament 	=> p_formapagament,
						   p_estatdespesa  	=> p_estatdespesa,
                     p_iban            => p_iban,
                     p_numtargeta      => p_numtargeta,
                     p_empresa         => NULL,
						   p_usuari        	=> p_usuari,
                     RSP               => RSP
						  );

			COMMIT;
			RSP :=  'OK';
		EXCEPTION
         WHEN ERR_PARAMS_NULL
         THEN
            RSP := PK_ERRORS.ERR_PARAMS_NULL;
         WHEN ERR_USER
         THEN
            RSP := PK_ERRORS.ERR_USER;
         WHEN ERR_CODI_DESPESA
         THEN
            RSP := PK_ERRORS.ERR_CODI_DESPESA;
         WHEN ERR_ESTAT_DESPESA
         THEN
            RSP := PK_ERRORS.ERR_ESTAT_DESPESA;
         WHEN ERR_FORMA_PAGAMENT
         THEN 
            RSP := PK_ERRORS.ERR_FORMA_PAGAMENT;
         WHEN ERR_IBAN
         THEN  
            RSP := PK_ERRORS.ERR_IBAN;
         WHEN ERR_CREDITCARD
         THEN
            RSP := PK_ERRORS.ERR_CREDITCARD;
         WHEN ERR_IBAN_CREDITCARD
         THEN
            RSP := PK_ERRORS.ERR_IBAN_CREDITCARD;
         WHEN ERR_IBRUT
         THEN
            RSP := PK_ERRORS.ERR_IBRUT;
         WHEN ERR_IVA    
         THEN
            RSP := PK_ERRORS.ERR_IVA;
			WHEN OTHERS 
         THEN
            IF RSP IS NULL
            THEN
               RSP := CONCAT(PK_ERRORS.ERR_NOTRACTAT, (TO_CHAR(SQLCODE) || ' '|| SUBSTR(SQLERRM,1,400)));
            END IF;
            ROLLBACK;
		END;
		p_log (v_nomproc, v_parin, RSP);
	END modif_despesa_parlament;
	
	PROCEDURE baixa_despesa (
		p_id			   IN	   despesa.id%TYPE,
		p_usuari		   IN    despesa.usubaixa%TYPE,
		RSP			   OUT   NOCOPY varchar2
	)  
   IS
		v_nomproc	logs.nomproc%TYPE := 'PK_ABM.baxia_despesa';
		v_parin		logs.parin%TYPE;
      
      v_exist           NUMBER;
      v_any             despesa.anyo%TYPE;
      v_inet            despesa.inet%TYPE;
      v_parlament       despesa.parlament%TYPE;
      v_parlamentari    despesa.parlamentari%TYPE;
      v_origenDespesa   despesa.origenDespesa%TYPE;
      v_estatDespesa    despesa.estatDespesa%TYPE;
      
      ERR_PARAMS_NULL         EXCEPTION;
      ERR_USER                EXCEPTION;
      ERR_BAIXA_DESPESA       EXCEPTION;
      
	BEGIN
		v_parin := CONCAT('p_id=', p_id);	
		v_parin := v_parin ||  CONCAT(' p_usuari=', p_usuari);
      
		BEGIN
         -- Validació del paràmetres d'entrada
         IF p_id IS NULL 
         THEN
            RAISE ERR_PARAMS_NULL;
         END IF;
         
         IF p_usuari IS NULL
         THEN
            RAISE ERR_USER;
         END IF;
         
         -- Comprova que la despesa existeix i que esta operativa
         BEGIN
            SELECT 1 INTO v_exist
              FROM despesa
             WHERE id = p_id
               AND databaixa IS NULL;
         EXCEPTION
            WHEN NO_DATA_FOUND
            THEN
               RAISE ERR_BAIXA_DESPESA;
         END;
         
         -- Dona de baixa la despesa i recupera les dades necessàries per cridar al procediment
         -- estadístic.
			UPDATE despesa
		      SET usubaixa = p_usuari,
			       databaixa = sysdate
          WHERE id = p_id
         RETURNING anyo, inet, parlament, parlamentari, origenDespesa, estatDespesa
              INTO v_any, v_inet, v_parlament, v_parlamentari, v_origenDespesa, v_estatDespesa;
         
         
         -- Crida al procediment estadístic per enregistrar l'operació. 
         PK_ESTADISTICA.baixa_despesa ( p_any           => v_any,
                                        p_inet          => v_inet,                                        
                                        p_parlament     => v_parlament,
                                        p_parlamentari  => v_parlamentari,
                                        p_origenDespesa => v_origenDespesa,
                                        p_estatDespesa  => v_estatDespesa,
                                        RSP             => RSP
                                      );
			COMMIT;
			RSP :=  'OK';
		EXCEPTION
         WHEN ERR_PARAMS_NULL
         THEN
            RSP := PK_ERRORS.ERR_PARAMS_NULL;
         WHEN ERR_USER
         THEN
            RSP := PK_ERRORS.ERR_USER;
         WHEN ERR_BAIXA_DESPESA
         THEN
            RSP := PK_ERRORS.ERR_BAIXA_DESPESA;
			WHEN OTHERS 
         THEN
            IF RSP IS NULL
            THEN
               RSP := CONCAT(PK_ERRORS.ERR_NOTRACTAT, (TO_CHAR(SQLCODE) || ' '|| SUBSTR(SQLERRM,1,400)));
            END IF;
            ROLLBACK;
		END;
		p_log (v_nomproc, v_parin, RSP);
   END baixa_despesa;
	
   PROCEDURE alta_despesa_parlamentari (
		p_codidespesa	 IN	despesa.codidespesa%TYPE,
		p_parlamentari	 IN	despesa.parlamentari%TYPE,
		p_data          IN	despesa.data%TYPE,
		p_ibrut         IN	despesa.ibrut%TYPE,
		p_iva           IN	despesa.iva%TYPE,
		p_descripcio    IN	despesa.descripcio%TYPE,
		p_justificacio  IN	despesa.justificacio%TYPE,
		p_urldocument   IN	despesa.urldocument%TYPE,
		p_formapagament IN	despesa.formapagament%TYPE,
		p_estatdespesa  IN	despesa.estatdespesa%TYPE,
      p_iban          IN   despesa.iban%TYPE,
      p_numtargeta    IN   despesa.numtargeta%TYPE,
		p_empresa	    IN despesa.empresa%TYPE,
		p_usuari		    IN	despesa.usualta%TYPE,
		RSP				 OUT	NOCOPY VARCHAR2
	)  IS
		v_nomproc	logs.nomproc%TYPE := 'PK_ABM.alta_despesa_parlamentari';
		v_parin		logs.parin%TYPE;
      
      v_exist        NUMBER;
      v_parlament    despesa.parlament%TYPE;
      v_partit       despesa.partit%TYPE;
      
      ERR_PARAMS_NULL            EXCEPTION;
      ERR_USER                   EXCEPTION;
      ERR_ESTAT_DESPESA          EXCEPTION;
      ERR_CODI_DESPESA           EXCEPTION;
      ERR_BAIXA_PARLAMENT        EXCEPTION;
      ERR_BAIXA_PARTIT           EXCEPTION;
      ERR_BAIXA_PARL             EXCEPTION;
      ERR_IBAN                   EXCEPTION;
      ERR_CREDITCARD             EXCEPTION;
      ERR_IBAN_CREDITCARD        EXCEPTION;
      ERR_IBRUT                  EXCEPTION;
      ERR_IVA                    EXCEPTION;
      ERR_FORMA_PAGAMENT         EXCEPTION;
      ERR_DESPESA_JUSTIFICACIO   EXCEPTION;
	BEGIN
		v_parin := CONCAT('p_codidespesa=', p_codidespesa);
      v_parin := v_parin || CONCAT('; p_parlamentari=', p_parlamentari);
      v_parin := v_parin || CONCAT('; p_data=', p_data);
      v_parin := v_parin || CONCAT('; p_ibrut=', p_ibrut);          
      v_parin := v_parin || CONCAT('; p_iva=', p_iva);      
      v_parin := v_parin || CONCAT('; p_descripcio=', p_descripcio);
      v_parin := v_parin || CONCAT('; p_justificacio=', p_justificacio);   
      v_parin := v_parin || CONCAT('; p_urldocument=', p_urldocument);
      v_parin := v_parin || CONCAT('; p_formapagament=', p_formapagament);  
      v_parin := v_parin || CONCAT('; p_estatdespesa=', p_estatdespesa);
      v_parin := v_parin || CONCAT('; p_iban=', p_iban);
      v_parin := v_parin || CONCAT('; p_numtargeta=', p_numtargeta);
      v_parin := v_parin || CONCAT('; p_empresa=', p_empresa);
      v_parin := v_parin || CONCAT('; p_usuari=', p_usuari);      
      
		BEGIN
         -- Validació dels paràmetres d'entrada
         IF p_codidespesa IS NULL OR p_parlamentari IS NULL
            OR p_data IS NULL OR p_ibrut IS NULL OR p_iva IS NULL
            OR p_descripcio IS NULL OR p_justificacio IS NULL 
            OR p_formapagament IS NULL OR p_estatdespesa IS NULL 
         THEN
            RAISE ERR_PARAMS_NULL;
         END IF;
         
         IF p_usuari IS NULL
         THEN
            RAISE ERR_USER;
         END IF;
         
         -- Recupera el parlament i partit actuals dels parlamentari
         SELECT parlament, partit
           INTO v_parlament, v_partit
           FROM parlamentari
          WHERE id = p_parlamentari;
         
         -- Valida que el parlament estigui operatiu
         IF NOT isParlamentOperatiu(v_parlament)
         THEN
            RAISE ERR_BAIXA_PARLAMENT;
         END IF;
         
         -- Valida que el partit estigui operatiu
         IF NOT isPartitOperatiu (v_partit)
         THEN
            RAISE ERR_BAIXA_PARTIT;
         END IF;
         
         -- Valia que el parlamentari estigui operatiu
         IF NOT isParlamentariOperatiu(p_parlamentari)
         THEN
            RAISE ERR_BAIXA_PARL;
         END IF;
         
         -- Mira que el codi de despesa existeixi i que pertany
         -- al tipus de parlamentari
         SELECT COUNT(1) INTO v_exist
           FROM tipusdespesaparlamentari
          WHERE codi = p_codidespesa;
          
         IF v_exist = 0 
         THEN
            RAISE ERR_CODI_DESPESA;
         END IF;
         
         -- Valida l'estat de la despesa
         SELECT COUNT(1) INTO v_exist
           FROM estatdespesa
          WHERE id = p_estatdespesa;
          
         IF v_exist = 0
         THEN
            RAISE ERR_ESTAT_DESPESA;
         END IF;
         
         -- Valida la forma de pagament de la despesa
         SELECT count(1) INTO v_exist
           FROM formapagament
          WHERE id = p_formapagament;
          
         IF v_exist = 0 
         THEN
            RAISE ERR_FORMA_PAGAMENT;
         END IF;         
         
         -- En el cas que la forma de pagament sigui per compte és necessari
         -- informar el IBAN, si la forma de pagament és targeta és necessari
         -- informar el número de targeta.
         -- En cap cas es pot informar el número de compte i el número de targeta
         IF p_formapagament = 1 AND p_iban IS NULL
         THEN
            RAISE ERR_IBAN;
         ELSIF p_formapagament = 2 AND p_numtargeta IS NULL
         THEN
            RAISE ERR_CREDITCARD;
         ELSIF p_iban IS NOT NULL AND p_numtargeta IS NOT NULL
         THEN
            RAISE ERR_IBAN_CREDITCARD;
         END IF;
         
         -- Comprova que l'import sigui superior a 0.
         IF p_ibrut <= 0 
         THEN
            RAISE ERR_IBRUT;
         END IF;
         
         -- Valida que l'iva no sigui negatiu
         IF p_iva < 0
         THEN
            RAISE ERR_IVA;
         END IF;                  
         
         -- Valida que en el cas que la despesa sigui superior a 30€ s'informi
         -- l'empresa i path dels documents escanetjats.
         IF p_ibrut > 30 AND (p_empresa IS NULL OR p_urldocument IS NULL)
         THEN
            RAISE ERR_DESPESA_JUSTIFICACIO;
         END IF;
         
         -- Crida al procediment intern que dona d'alta la despesa
			alta_despesa ( p_origendespesa => 2,
                        p_codidespesa		=> p_codidespesa,
                        p_parlament     	=> v_parlament,
                        p_parlamentari	   => p_parlamentari,
                        p_partit			   => v_partit,
                        p_data         	=> p_data,
                        p_ibrut         	=> p_ibrut,
                        p_iva           	=> p_iva,
                        p_descripcio    	=> p_descripcio,
                        p_justificacio  	=> p_justificacio,
                        p_urldocument   	=> p_urldocument,
                        p_formapagament 	=> p_formapagament,
                        p_estatdespesa  	=> p_estatdespesa,
                        p_iban            => p_iban,
                        p_numtargeta      => p_numtargeta,
                        p_empresa         => p_empresa,
                        p_usuari        	=> p_usuari,
                        RSP               => RSP
						 );
			COMMIT;
			RSP :=  'OK';
		EXCEPTION
         WHEN ERR_PARAMS_NULL
         THEN
            RSP := PK_ERRORS.ERR_PARAMS_NULL;
         WHEN ERR_USER
         THEN
            RSP := PK_ERRORS.ERR_USER;
         WHEN ERR_BAIXA_PARLAMENT
         THEN
            RSP := PK_ERRORS.ERR_BAIXA_PARLAMENT;
         WHEN ERR_BAIXA_PARTIT
         THEN
            RSP := PK_ERRORS.ERR_BAIXA_PARTIT;
         WHEN ERR_BAIXA_PARL 
         THEN
            RSP := PK_ERRORS.ERR_BAIXA_PARL;
         WHEN ERR_CODI_DESPESA
         THEN
            RSP := PK_ERRORS.ERR_CODI_DESPESA;
         WHEN ERR_ESTAT_DESPESA
         THEN
            RSP := PK_ERRORS.ERR_ESTAT_DESPESA;
         WHEN ERR_FORMA_PAGAMENT
         THEN
            RSP := PK_ERRORS.ERR_FORMA_PAGAMENT;
         WHEN ERR_IBAN
         THEN
            RSP := PK_ERRORS.ERR_IBAN;
         WHEN ERR_CREDITCARD
         THEN
            RSP :=  PK_ERRORS.ERR_CREDITCARD;
         WHEN ERR_IBAN_CREDITCARD
         THEN
            RSP := PK_ERRORS.ERR_IBAN_CREDITCARD;
         WHEN ERR_IBRUT
         THEN
            RSP := PK_ERRORS.ERR_IBRUT;
         WHEN ERR_IVA
         THEN
            RSP := PK_ERRORS.ERR_IVA;
         WHEN ERR_DESPESA_JUSTIFICACIO
         THEN
            RSP := PK_ERRORS.ERR_DESPESA_JUSTIFICACIO;
			WHEN OTHERS 
         THEN
            IF RSP IS NULL
            THEN
               RSP := CONCAT(PK_ERRORS.ERR_NOTRACTAT, (TO_CHAR(SQLCODE) || ' '|| SUBSTR(SQLERRM,1,400)));
            END IF;
            ROLLBACK;
		END;
		p_log (v_nomproc, v_parin, RSP);
	END alta_despesa_parlamentari;
	
	PROCEDURE modif_despesa_parlamentari (
		p_id			    IN	despesa.id%TYPE,
		p_codidespesa	 IN	despesa.codidespesa%TYPE,
		p_data          IN	despesa.data%TYPE,
		p_ibrut         IN	despesa.ibrut%TYPE,
		p_iva           IN	despesa.iva%TYPE,
		p_descripcio    IN	despesa.descripcio%TYPE,
		p_justificacio  IN	despesa.justificacio%TYPE,
		p_urldocument   IN	despesa.urldocument%TYPE,
		p_formapagament IN	despesa.formapagament%TYPE,
		p_estatdespesa  IN	despesa.estatdespesa%TYPE,
      p_iban          IN   despesa.iban%TYPE,
      p_numtargeta    IN   despesa.numtargeta%TYPE,
		p_empresa  	  IN despesa.empresa%TYPE,
		p_usuari		    IN	despesa.usualta%TYPE,      
		RSP				 OUT	NOCOPY VARCHAR2
	)  IS
		v_nomproc	logs.nomproc%TYPE := 'PK_ABM.modif_despesa_parlamentari';
		v_parin		logs.parin%TYPE;
      
      v_exist        NUMBER;
      
      ERR_PARAMS_NULL            EXCEPTION;
      ERR_USER                   EXCEPTION;
      ERR_CODI_DESPESA           EXCEPTION;
      ERR_ESTAT_DESPESA          EXCEPTION;
      ERR_FORMA_PAGAMENT         EXCEPTION;
      ERR_IBAN                   EXCEPTION;
      ERR_CREDITCARD             EXCEPTION;
      ERR_IBAN_CREDITCARD        EXCEPTION;
      ERR_IBRUT                  EXCEPTION;
      ERR_IVA                    EXCEPTION;
      ERR_DESPESA_JUSTIFICACIO   EXCEPTION;
	BEGIN
		v_parin := CONCAT('p_id=', p_id);
		v_parin := v_parin || CONCAT('; p_codidespesa=', p_codidespesa);
		v_parin := v_parin || CONCAT('; p_data=', p_data);
		v_parin := v_parin || CONCAT('; p_ibrut=', p_ibrut);
		v_parin := v_parin || CONCAT('; p_iva=', p_iva);
		v_parin := v_parin || CONCAT('; p_descripcio=', p_descripcio);
		v_parin := v_parin || CONCAT('; p_justificacio=', p_justificacio);
		v_parin := v_parin || CONCAT('; p_urldocument=', p_urldocument);
		v_parin := v_parin || CONCAT('; p_formapagament=', p_formapagament);
		v_parin := v_parin || CONCAT('; p_estatdespesa=', p_estatdespesa);
      v_parin := v_parin || CONCAT('; p_iban=', p_iban);
      v_parin := v_parin || CONCAT('; p_numtargeta=', p_numtargeta);
      v_parin := v_parin || CONCAT('; p_empresa=', p_empresa);
		v_parin := v_parin || CONCAT('; p_usuari=', p_usuari);
		
      BEGIN
         -- Validació dels paràmetres d'entrada
         IF p_id IS NULL OR p_codidespesa IS NULL OR p_data IS NULL
            OR p_ibrut IS NULL OR p_iva IS NULL OR p_descripcio IS NULL
            OR p_justificacio IS NULL
            OR p_formapagament IS NULL OR p_estatdespesa IS NULL
         THEN
            RAISE ERR_PARAMS_NULL;
         END IF;
         
         IF p_usuari IS NULL
         THEN
            RAISE ERR_USER;
         END IF;
         
         SELECT COUNT(1) INTO v_exist
           FROM tipusdespesaparlamentari
          WHERE codi = p_codidespesa;
          
         IF v_exist = 0 
         THEN
            RAISE ERR_CODI_DESPESA;
         END IF;
         
         SELECT COUNT(1) INTO v_exist
           FROM estatdespesa
          WHERE id = p_estatdespesa;
          
         IF v_exist = 0
         THEN
            RAISE ERR_ESTAT_DESPESA;
         END IF;
         
         SELECT count(1) INTO v_exist
           FROM formapagament
          WHERE id = p_formapagament;
          
         IF v_exist = 0 
         THEN
            RAISE ERR_FORMA_PAGAMENT;
         END IF;         
         
         -- En el cas que la forma de pagament sigui per compte és necessari
         -- informar el IBAN, si la forma de pagament és targeta és necessari
         -- informar el número de targeta.
         -- En cap cas es pot informar el número de compte i el número de targeta
         IF p_formapagament = 1 AND p_iban IS NULL
         THEN
            RAISE ERR_IBAN;
         ELSIF p_formapagament = 2 AND p_numtargeta IS NULL
         THEN
            RAISE ERR_CREDITCARD;
         ELSIF p_iban IS NOT NULL AND p_numtargeta IS NOT NULL
         THEN
            RAISE ERR_IBAN_CREDITCARD;
         END IF;
         
         -- Valida que l'import sigui superior a 0.
         IF p_ibrut <= 0 
         THEN
            RAISE ERR_IBRUT;
         END IF;
         
         -- Valida que l'iva no sigui negatiu
         IF p_iva < 0
         THEN
            RAISE ERR_IVA;
         END IF;         
         
         -- Comprova que s'informi l'empresa i l'URL dels document en el cas
         -- que l'import brutu de la despesa superi els 30 €
         IF p_ibrut > 30
            AND (p_empresa IS NULL OR p_urldocument IS NULL)
         THEN
            RAISE ERR_DESPESA_JUSTIFICACIO;
         END IF;
         
         -- Crida al procediment intern que enregistre la modificació
         modif_despesa (p_id				   => p_id,
                        p_codidespesa	   => p_codidespesa,
                        p_data         	=> p_data,
                        p_ibrut         	=> p_ibrut,
                        p_iva           	=> p_iva,
                        p_descripcio    	=> p_descripcio,
                        p_justificacio  	=> p_justificacio,
                        p_urldocument   	=> p_urldocument,
                        p_formapagament 	=> p_formapagament,
                        p_estatdespesa  	=> p_estatdespesa,
                        p_iban            => p_iban,
                        p_numtargeta      => p_numtargeta,
                        p_empresa         => p_empresa,
                        p_usuari        	=> p_usuari,
                        RSP               => RSP
                      );

			COMMIT;
			RSP :=  'OK';
		EXCEPTION
         WHEN ERR_PARAMS_NULL
         THEN
            RSP := PK_ERRORS.ERR_PARAMS_NULL;
         WHEN ERR_USER
         THEN
            RSP := PK_ERRORS.ERR_USER;
         WHEN ERR_CODI_DESPESA
         THEN
            RSP := PK_ERRORS.ERR_CODI_DESPESA;
         WHEN ERR_ESTAT_DESPESA
         THEN
            RSP := PK_ERRORS.ERR_ESTAT_DESPESA;
         WHEN ERR_FORMA_PAGAMENT
         THEN 
            RSP := PK_ERRORS.ERR_FORMA_PAGAMENT;
         WHEN ERR_IBAN
         THEN  
            RSP := PK_ERRORS.ERR_IBAN;
         WHEN ERR_CREDITCARD
         THEN
            RSP := PK_ERRORS.ERR_CREDITCARD;
         WHEN ERR_IBAN_CREDITCARD
         THEN
            RSP := PK_ERRORS.ERR_IBAN_CREDITCARD;
         WHEN ERR_IBRUT
         THEN
            RSP := PK_ERRORS.ERR_IBRUT;
         WHEN ERR_IVA    
         THEN
            RSP := PK_ERRORS.ERR_IVA;
			WHEN ERR_DESPESA_JUSTIFICACIO
         THEN
            RSP := PK_ERRORS.ERR_DESPESA_JUSTIFICACIO;
         WHEN OTHERS 
         THEN
            IF RSP IS NULL
            THEN
               RSP := CONCAT(PK_ERRORS.ERR_NOTRACTAT, (TO_CHAR(SQLCODE) || ' '|| SUBSTR(SQLERRM,1,400)));
            END IF;
            ROLLBACK;
		END;
		p_log (v_nomproc, v_parin, RSP);
	END modif_despesa_parlamentari;
	
END;
/
