CREATE OR REPLACE PACKAGE BODY PK_ESTADISTICA 
AS
   /**
   * <p>Procediment intern que calcula l'estadística :
   * "Donat un any concret: la suma de totes les despeses de tots els parlaments 
   * (incloent les despeses generals dels parlaments i les despeses associades 
   * als parlamentaris)" </p>
   * @author   Toni Torres
   *
   * @param p_any       NUMBER. Any de la despesa
   * @param p_import    NUMBER. Import de la despesa
   * @param RSP                  Resposta de l'operació
   *                             <ul>"OK" = Procés executat correctament</ul>
   *                             <ul>"ERROR:" = Procés finalitzat amb error i el motiu de l'error</ul>
   */
   PROCEDURE stat_EstTotal (     
      p_any          IN    EstTotal.anyo%TYPE,
      p_import       IN    Despesa.inet%TYPE,
      RSP            OUT   NOCOPY VARCHAR2
   )
   IS
      v_nomproc	      logs.nomproc%TYPE := 'PK_ESTADISTICA.stat_EstTotal';
		v_parin		      logs.parin%TYPE;
      v_total           NUMBER;
   BEGIN
      v_parin := CONCAT('p_any=', p_any);
      v_parin := v_parin || CONCAT('; p_import=', p_import);
      RSP := 'OK';

      -- Modifica les dades de la taula, l'import pot ser negatiu en el cas
      -- que sigui una baixa o modificació de despesa.
      -- En el cas de modificació primer es descomptarà l'import antig i després
      -- es sumarà el nou dins l'any corresponent.
      UPDATE EstTotal
         SET total = total + p_import
       WHERE anyo = p_any;
      
      -- En el cas de no modificar cap registre realitza l'inseció
      IF SQL%ROWCOUNT = 0 THEN
         INSERT INTO EstTotal (anyo, total) VALUES (p_any, p_import);
      END IF;
      
      p_log (v_nomproc, v_parin, RSP);
   EXCEPTION
      WHEN OTHERS THEN
         -- Enregistra l'error i el propaga.
         RSP := CONCAT(PK_ERRORS.ERR_STAT_ESTTOTAL , (TO_CHAR(SQLCODE) || ' '|| SUBSTR(SQLERRM,1,400)));
         p_log (v_nomproc, v_parin, RSP);
         RAISE;
   END stat_EstTotal;
   
   /**
   * <p>Procediment intern que calcula l'estadística :
   * "Donat un any concret: el nom del parlament amb una mitjana de 
   * despesa associada als seus parlamentaris més alta, i el valor 
   * d'aquesta mitjana de despesa" </p>
   * @author   Toni Torres
   *
   * @param p_any       NUMBER. Any de la despesa
   * @param RSP                  Resposta de l'operació
   *                             <ul>"OK" = Procés executat correctament</ul>
   *                             <ul>"ERROR:" = Procés finalitzat amb error i el motiu de l'error</ul>
   */
   PROCEDURE stat_EstMitjaParlament (
      p_any          IN    EstMitjaParlament.anyo%TYPE,
      RSP            OUT   NOCOPY VARCHAR2      
   )
   IS
      v_nomproc	      logs.nomproc%TYPE := 'PK_ESTADISTICA.stat_EstMitjaParlament';
		v_parin		      logs.parin%TYPE;
      
      v_mitja           EstMitjaParlament.mitjana%TYPE;
      v_parlament       EstMitjaParlament.parlament%TYPE;
      v_nom             EstMitjaParlament.nom%TYPE;
   BEGIN
   
      v_parin := CONCAT('p_any=', p_any);
      RSP := 'OK';
      
      -- recupera la mitja de parlamentaris per l'any
      SELECT AVG (inet) INTO v_mitja
        FROM despesaStat         
       WHERE anyo = p_any
         AND origendespesa = 2;
      
      BEGIN
         SELECT idParlament, nomParlament
           INTO v_parlament, v_nom
           FROM ( SELECT p.id idParlament, p.nom nomParlament
                    FROM despesaStat d INNER JOIN parlament p ON (d.parlament = p.id)
                   WHERE d.anyo = p_any
                     AND d.origendespesa = 2
                  HAVING sum(inet) >= v_mitja
                  GROUP BY p.id, p.nom
                  ORDER BY sum(inet) DESC)        
          WHERE ROWNUM < 2;
         
         UPDATE EstMitjaParlament
            SET parlament = v_parlament,
                nom = v_nom,
                mitjana = v_mitja
          WHERE anyo = p_any;
         
         -- En el cas de no modificar cap dada dona d'alta el registre
         IF SQL%ROWCOUNT = 0 
         THEN
            INSERT INTO EstMitjaParlament (anyo, parlament, nom, mitjana)
               VALUES (p_any, v_parlament, v_nom, v_mitja);
         END IF;
      EXCEPTION
         WHEN NO_DATA_FOUND
         THEN
            DELETE FROM EstMitjaParlament
             WHERE anyo = p_any;
      END;
      
      p_log(v_nomproc, v_parin, RSP);
   EXCEPTION
      WHEN OTHERS THEN
         -- En cas d'error, l'enregistra i propaga l'excepció
         RSP := CONCAT(PK_ERRORS.ERR_STAT_ESTMITJAPARLAMENT, (TO_CHAR(SQLCODE) || ' '|| SUBSTR(SQLERRM,1,400)));
         p_log(v_nomproc, v_parin, RSP);
         RAISE;
   END stat_EstMitjaParlament;
   
   /**
   * <p>Procediment intern que calcula l'estadística :
   * "Donat un any concret: el parlamentari que ha tingut més despesa 
   * associada i el que menys (tenint en compte tots els 
   * parlaments)" </p>
   * @author   Toni Torres
   *
   * @param p_any       NUMBER. Any de la despesa
   * @param RSP                  Resposta de l'operació
   *                             <ul>"OK" = Procés executat correctament</ul>
   *                             <ul>"ERROR:" = Procés finalitzat amb error i el motiu de l'error</ul>
   */
   PROCEDURE stat_EstParlamentarisOposats (
      p_any             IN    EstParlamentarisOposats.anyo%TYPE,
      RSP               OUT   NOCOPY VARCHAR2
   )
   IS   
      v_nomproc	      logs.nomproc%TYPE := 'PK_ESTADISTICA.stat_EstParlamentarisOposats';
		v_parin		      logs.parin%TYPE;

      v_parlamentariMenys        EstParlamentarisOposats.parlamentariMenys%TYPE;
      v_nomMenys                 EstParlamentarisOposats.nomMenys%TYPE;
      v_despesaMenys             EstParlamentarisOposats.despesaMenys%TYPE;
      v_parlamentariMes          EstParlamentarisOposats.parlamentariMes%TYPE;
      v_nomMes                   EstParlamentarisOposats.nomMes%TYPE;
      v_despesaMes               EstParlamentarisOposats.despesaMes%TYPE;
   BEGIN
      v_parin := CONCAT('p_any=', p_any);
      RSP := 'OK';
      
      BEGIN
         SELECT idParlamentari, nomParlamentari, total 
           INTO v_parlamentariMenys, v_nomMenys, v_despesaMenys
           FROM ( SELECT p.id idParlamentari, p.nom || ' ' || p.cognoms nomParlamentari, sum(d.inet) TOTAL
                    FROM despesaStat d INNER JOIN parlamentari p ON (d.parlamentari = p.id)
                   WHERE d.anyo = p_any
                     AND d.origendespesa = 2
                   GROUP BY p.id, p.nom || ' ' || p.cognoms
                   ORDER BY TOTAL ASC)        
          WHERE ROWNUM < 2;
   
         SELECT idParlamentari, nomParlamentari, total
           INTO v_parlamentariMes, v_nomMes, v_despesaMes
           FROM ( SELECT p.id idParlamentari, p.nom || ' ' || p.cognoms nomParlamentari, sum(d.inet) TOTAL
                    FROM despesaStat d INNER JOIN parlamentari p ON (d.parlamentari = p.id)
                   WHERE d.anyo = p_any
                     AND d.origendespesa = 2
                   GROUP BY p.id, p.nom || ' ' || p.cognoms
                   ORDER BY TOTAL DESC)        
          WHERE ROWNUM < 2;
      
         UPDATE EstParlamentarisOposats
            SET parlamentariMenys = v_parlamentariMenys,
                nomMenys = v_nomMenys,
                despesaMenys = v_despesaMenys,
                parlamentariMes = v_parlamentariMes,
                nomMes = v_nomMes,
                despesaMes = v_despesaMes
          WHERE anyo = p_any;
         -- En el cas de no modificar cap registre el dona d'alta
         IF SQL%ROWCOUNT = 0 
         THEN
            INSERT INTO EstParlamentarisOposats (anyo, parlamentariMenys, nomMenys, despesaMenys,
                  parlamentariMes, nomMes, despesaMes)
               VALUES (p_any, v_parlamentariMenys, v_nomMenys, v_despesaMenys,
                  v_parlamentariMes, v_nomMes, v_despesaMes);
         END IF;
      EXCEPTION
         WHEN NO_DATA_FOUND
         THEN
            -- Si no troba cap dada per l'any indicat esborra el contingut de la taula
            -- per l'any indicat.
            DELETE FROM EstParlamentarisOposats
             WHERE anyo = p_any;
      END;
      p_log(v_nomproc, v_parin, RSP);
   EXCEPTION
      WHEN OTHERS 
      THEN
         -- Registra l'error i propaga l'excepció
         RSP := CONCAT(PK_ERRORS.ERR_STAT_ESTPARMENTARISOPOSATS, (TO_CHAR(SQLCODE) || ' '|| SUBSTR(SQLERRM,1,400)));
         p_log (v_nomproc, v_parin, RSP);
         RAISE;
   END stat_EstParlamentarisOposats;
   
   /**
   * <p>Procediment intern que calcula l'estadística :
   * "Donat un any concret: la mitjana de despesa d'un parlamentari 
   * durant aquell any (tenint en compte tots els parlamentaris de 
   * tots els parlaments)" </p>
   * @author   Toni Torres
   *
   * @param p_any       NUMBER. Any de la despesa
   * @param RSP                  Resposta de l'operació
   *                             <ul>"OK" = Procés executat correctament</ul>
   *                             <ul>"ERROR:" = Procés finalitzat amb error i el motiu de l'error</ul>
   */
   PROCEDURE stat_EstMitjanaParlamentari (
      p_any             IN    EstMitjanaParlamentari.anyo%TYPE,
      RSP               OUT   NOCOPY VARCHAR2
   )
   IS
      v_nomproc	      logs.nomproc%TYPE := 'PK_ESTADISTICA.stat_EstMitjanaParlamentari';
		v_parin		      logs.parin%TYPE;

      v_mitjana         estmitjanaparlamentari.mitjana%TYPE;
      v_numparla        NUMBER;
   BEGIN
      v_parin := CONCAT('p_any=', p_any);
      RSP := 'OK';
      
      -- Suma les despeses del parlamentaris per l'any indicat
      SELECT NVL(SUM(inet),0) INTO v_mitjana
        FROM despesaStat
       WHERE origendespesa = 2
         AND anyo = p_any;
      
      -- Si l'any indicat és el mateix que l'any en curs agafa el nombre
      -- de parlamentaris de la taula parlament, sino mira quans parlamentaris
      -- hi havia assignats a aquell parlament
      IF p_any = EXTRACT (YEAR FROM SYSDATE)
      THEN
      	SELECT SUM(numparlamentaris)
      	  INTO v_numparla
      	  FROM parlament;
      ELSE
      	SELECT COUNT(1)
      	  INTO v_numparla
      	  FROM parlamentari
      	 WHERE EXTRACT(YEAR FROM dataalta) <= p_any
      	   AND (EXTRACT(YEAR FROM databaixa) >= p_any
      	   		OR databaixa IS NULL);  
      END IF;
      
      v_mitjana := v_mitjana / v_numparla;
      
      UPDATE EstMitjanaParlamentari
         SET mitjana = v_mitjana
       WHERE anyo = p_any;
      
      -- Dona d'alta el registra sino se n'ha modificat cap
      IF SQL%ROWCOUNT = 0
      THEN
         INSERT INTO EstMitjanaParlamentari (anyo, mitjana)
               VALUES (p_any, v_mitjana);
      END IF;
      
      p_log (v_nomproc, v_parin, RSP);
   EXCEPTION
      WHEN OTHERS
      THEN
         -- Registra l'error i propaga l'excepció
         RSP := CONCAT(PK_ERRORS.ERR_STAT_ESTMITJANAPARMENTARI, (TO_CHAR(SQLCODE) || ' '|| SUBSTR(SQLERRM,1,400)));
         p_log (v_nomproc, v_parin, RSP);
         RAISE;
   END stat_EstMitjanaParlamentari;
   
   /**
   * <p>Procediment intern que calcula l'estadística :
   * "Donat un parlamentari: la despesa total que ha tingut els darrers 4 anys" </p>
   * @author   Toni Torres
   *
   * @param p_idparlamentari  NUMBER. Identificador del parlamentari
   * @param RSP                  Resposta de l'operació
   *                             <ul>"OK" = Procés executat correctament</ul>
   *                             <ul>"ERROR:" = Procés finalitzat amb error i el motiu de l'error</ul>
   */
   PROCEDURE stat_EstParlamentariAnys (
      p_idParlamentari  IN    EstParlamentariAnys.parlamentari%TYPE,
      RSP               OUT   NOCOPY VARCHAR2
   )
   IS
      v_nomproc	      logs.nomproc%TYPE := 'PK_ESTADISTICA.stat_EstParlamentariAnys';
		v_parin		      logs.parin%TYPE;
      
      v_total           EstParlamentariAnys.total%TYPE;
      v_parlamentari    EstParlamentariAnys.parlamentari%TYPE;
      v_nom             EstParlamentariAnys.nom%TYPE;
   BEGIN
      v_parin := CONCAT('p_idParlamentari=', p_idParlamentari);
      RSP := 'OK';
      
      -- Suma les despeses del parlamentari dels darrers 4 anys
      SELECT NVL(SUM (inet), 0)
        INTO v_total
        FROM despesaStat
       WHERE origendespesa = 2
         AND anyo > EXTRACT (YEAR FROM SYSDATE)-4
         AND parlamentari = p_idParlamentari;
      
      UPDATE EstParlamentariAnys
         SET total = v_total
       WHERE parlamentari = p_idParlamentari;
      
      -- Si no s'ha modificat cap registre, cerca el nom del parlamentari
      -- i dona d'alta el registre
      IF SQL%ROWCOUNT = 0 
      THEN
         SELECT nom || ' ' || cognoms 
           INTO v_nom
           FROM parlamentari
          WHERE id = p_idParlamentari;
          
         INSERT INTO EstParlamentariAnys (parlamentari, nom, total)
            VALUES (p_idParlamentari, v_nom, v_total);      
      END IF;
      p_log (v_nomproc, v_parin, RSP);
   EXCEPTION
      WHEN OTHERS
      THEN
         -- Registra l'error i propaga l'excepció
         RSP := CONCAT(PK_ERRORS.ERR_STAT_ESTPARLAMENTARIANYS, (TO_CHAR(SQLCODE) || ' '|| SUBSTR(SQLERRM,1,400)));
         p_log (v_nomproc, v_parin, RSP);
         RAISE;
   END stat_EstParlamentariAnys;
   
   /**
   * <p>Procediment intern que calcula l'estadística :
   * "Donat un any concret: el parlament que ha tingut un total de 
   * despeses més alt (incloent les despeses generals dels parlaments i 
   * les despeses associades als parlamentaris)" </p>
   * @author   Toni Torres
   *
   * @param p_any  NUMBER. Any de la despesa
   * @param RSP                  Resposta de l'operació
   *                             <ul>"OK" = Procés executat correctament</ul>
   *                             <ul>"ERROR:" = Procés finalitzat amb error i el motiu de l'error</ul>
   */
   PROCEDURE stat_EstParlamentMesAlta (
      p_any          IN    EstParlamentMesAlta.anyo%TYPE,
      RSP            OUT   NOCOPY VARCHAR2
   )
   IS      
      v_nomproc	      logs.nomproc%TYPE := 'PK_ESTADISTICA.stat_EstParlamentMesAlta';
		v_parin		      logs.parin%TYPE;
      
      v_parlament       EstParlamentMesAlta.parlament%TYPE;
      v_nom             EstParlamentMesAlta.nom%TYPE;
      v_total           EstParlamentMesAlta.total%TYPE;
   BEGIN
      v_parin := CONCAT('p_any=', p_any);
      RSP := 'OK';
      
      BEGIN
         SELECT idParlament, nomParlament, total 
           INTO v_parlament, v_nom, v_total
           FROM ( SELECT p.id idParlament, p.nom nomParlament, SUM(d.inet) total
                    FROM despesaStat d INNER JOIN parlament p ON (d.parlament = p.id)
                   WHERE d.anyo = p_any
                   GROUP BY p.id, p.nom
                   ORDER BY total DESC)        
          WHERE rownum < 2;
          
         UPDATE EstParlamentMesAlta
            SET parlament = v_parlament,
                nom = v_nom,
                total = v_total
          WHERE anyo = p_any;
         
         -- En el cas de modificar cap registre el dona d'alta
         IF SQL%ROWCOUNT = 0 
         THEN
            INSERT INTO EstParlamentMesAlta (anyo, parlament, nom, total)
               VALUES (p_any, v_parlament, v_nom, v_total);
         END IF;
      EXCEPTION
         WHEN NO_DATA_FOUND 
         THEN
            -- Si no troba cap registre esborra les dades de la taula per aquell any
            DELETE FROM EstParlamentMesAlta
             WHERE anyo = p_any;
      END;
      p_log (v_nomproc, v_parin, RSP);
   EXCEPTION
      WHEN OTHERS 
      THEN
         -- Registra l'error i el propaga
         RSP := CONCAT(PK_ERRORS.ERR_STAT_ESTPARLAMENTMESALTA, (TO_CHAR(SQLCODE) || ' '|| SUBSTR(SQLERRM,1,400)));
         p_log (v_nomproc, v_parin, RSP);
         RAISE;
   END stat_EstParlamentMesAlta;
   
   /**
   * <p>Procediment intern que calcula l'estadística :
   * "Donat un parlament i any: la diferencia en euros entre el 
   * parlamentari que més despeses associades té i el que menys" </p>
   * @author   Toni Torres
   *
   * @param p_any          NUMBER. Any de la despesa
   * @param p_idPalament   NUMBER. Identificador del parlament
   * @param RSP                  Resposta de l'operació
   *                             <ul>"OK" = Procés executat correctament</ul>
   *                             <ul>"ERROR:" = Procés finalitzat amb error i el motiu de l'error</ul>
   */
   PROCEDURE stat_EstParlamentParlamentari (
      p_any             IN    EstParlamentParlamentari.anyo%TYPE,
      p_idParlament     IN    EstParlamentParlamentari.parlament%TYPE,
      RSP               OUT   NOCOPY VARCHAR2
   )
   IS
      v_nomproc	      logs.nomproc%TYPE := 'PK_ESTADISTICA.stat_EstParlamentParlamentari';
		v_parin		      logs.parin%TYPE;
   
      v_parlamentariMenys        EstParlamentParlamentari.parlamentariMenys%TYPE;
      v_nomMenys                 EstParlamentParlamentari.nomMenys%TYPE;
      v_totalMenys               EstParlamentParlamentari.totalMenys%TYPE;
      v_parlamentariMes          EstParlamentParlamentari.parlamentariMes%TYPE;
      v_nomMes                   EstParlamentParlamentari.nomMes%TYPE;
      v_totalMes                 EstParlamentParlamentari.totalMes%TYPE;
   BEGIN
      v_parin := CONCAT('p_any=', p_any);
      v_parin := v_parin || CONCAT ('; p_idParlament=', p_idParlament);
      RSP := 'OK';
   
      BEGIN
         SELECT idParlamentari, nomParlamentari, total 
           INTO v_parlamentariMenys, v_nomMenys, v_totalMenys
           FROM (SELECT p.id idParlamentari, p.nom || ' ' || p.cognoms nomParlamentari, sum(d.inet) Total
                   FROM despesaStat d INNER JOIN parlamentari p ON (d.parlamentari = p.id)
                  WHERE d.anyo = p_any
                    AND d.origenDespesa = 2
                    AND d.parlament = p_idParlament                 
                  GROUP BY p.id, p.nom || ' ' || p.cognoms
                  ORDER BY total ASC)
          WHERE ROWNUM < 2;
         
         SELECT idParlamentari, nomParlamentari, total 
           INTO v_parlamentariMes, v_nomMes, v_totalMes
           FROM (SELECT p.id idParlamentari, p.nom || ' ' || p.cognoms nomParlamentari, sum(d.inet) Total
                   FROM despesaStat d INNER JOIN parlamentari p ON (d.parlamentari = p.id)
                  WHERE d.anyo = p_any
                    AND d.origenDespesa = 2
                    AND p.parlament = p_idParlament
                  GROUP BY p.id, p.nom || ' ' || p.cognoms
                  ORDER BY total DESC)
          WHERE ROWNUM < 2;
          
         UPDATE EstParlamentParlamentari
            SET parlamentariMenys = v_parlamentariMenys,
                nomMenys = v_nomMenys,
                totalMenys = v_totalMenys,             
                parlamentariMes = v_parlamentariMes,
                nomMes = v_nomMes,
                totalMes = v_totalMes,
                diferencia = (v_totalMes - v_totalMenys)
          WHERE anyo = p_any
            AND parlament = p_idParlament;
         
         -- En el cas de no modificar el registre el dona d'alta
         IF SQL%ROWCOUNT = 0 
         THEN
            INSERT INTO EstParlamentParlamentari (anyo, parlament, parlamentariMenys, 
                                                  nomMenys, totalMenys, parlamentariMes,
                                                  nomMes, totalMes, diferencia)
                                          VALUES (p_any, p_idParlament, v_parlamentariMenys,
                                                  v_nomMenys, v_totalMenys, v_parlamentariMes,
                                                  v_nomMes, v_totalMes, (v_totalMes - v_totalMenys));
         END IF;
      EXCEPTION
         WHEN NO_DATA_FOUND
         THEN
            DELETE FROM EstParlamentParlamentari
             WHERE anyo = p_any AND parlament = p_idParlament;
      END;
      p_log (v_nomproc, v_parin, RSP);
   EXCEPTION
      WHEN OTHERS
      THEN
         -- Registra l'error i el propaga
         RSP := CONCAT(PK_ERRORS.ERR_STAT_ESTPARMENTPARMENTARI, (TO_CHAR(SQLCODE) || ' '|| SUBSTR(SQLERRM,1,400)));
         p_log (v_nomproc, v_parin, RSP);
         RAISE;
   END stat_EstParlamentParlamentari;
   
   /**
   * <p>Procediment intern que calcula l'estadística :
   * "Donat un parlament: la suma de totes les despeses dels darrers 4 
   * anys (incloent les despeses generals dels parlaments i les despeses 
   * associades als parlamentaris)" </p>
   * @author   Toni Torres
   *
   * @param p_idPalament   NUMBER. Identificador del parlament
   * @param RSP                  Resposta de l'operació
   *                             <ul>"OK" = Procés executat correctament</ul>
   *                             <ul>"ERROR:" = Procés finalitzat amb error i el motiu de l'error</ul>
   */
   PROCEDURE stat_EstParlamentAnys (
      p_idParlament     IN    EstParlamentAnys.parlament%TYPE,
      RSP               OUT   NOCOPY VARCHAR2
   )
   IS
      v_nomproc	      logs.nomproc%TYPE := 'PK_ESTADISTICA.stat_EstParlamentAnys';
		v_parin		      logs.parin%TYPE;
      
      v_total           EstParlamentAnys.total%TYPE;
      v_parlament       EstParlamentAnys.parlament%TYPE;
      v_nom             EstParlamentAnys.nom%TYPE;
   BEGIN
      v_parin := CONCAT('p_idParlament=', p_idParlament);
      RSP := 'OK';
      
      -- Suma les despeses del parlament dels darrer 4 anys.
      SELECT NVL(SUM (inet),0)
        INTO v_total
        FROM despesaStat
       WHERE anyo > EXTRACT (YEAR FROM SYSDATE)-4
         AND parlament = p_idParlament;
      
      UPDATE EstParlamentAnys
         SET total = v_total
       WHERE parlament = p_idParlament;
      
      -- Dona d'alta el registre si no existeix
      IF SQL%ROWCOUNT = 0 
      THEN
         SELECT nom 
           INTO v_nom
           FROM parlament
          WHERE id = p_idParlament;
          
         INSERT INTO EstParlamentAnys (parlament, nom, total)
            VALUES (p_idParlament, v_nom, v_total);      
      END IF;
      p_log (v_nomproc, v_parin, RSP);
   EXCEPTION
      WHEN OTHERS
      THEN
         -- Registra l'error i el propaga
         RSP := CONCAT(PK_ERRORS.ERR_STAT_ESTPARLAMENTANYS, (TO_CHAR(SQLCODE) || ' '|| SUBSTR(SQLERRM,1,400)));
         p_log (v_nomproc, v_parin, RSP);
         RAISE;
   END stat_EstParlamentAnys;
   
    /**
   * <p>Procediment que crida als procediments interns per calcular 
   * les estadístiques. </p>
   * @author   Toni Torres
   *
   * @param p_any             NUMBER. Any de la despesa
   * @param p_inet            NUMBER. Import net de la despesa
   * @param p_idPalament      NUMBER. Identificador del parlament
   * @param p_idPalamentari   NUMBER. Identificador del parlamentari
   * @param p_origenDespesa   NUMBER. Origen despesa. 1 Parlament; 2 Parlamentari
   * @param RSP                  Resposta de l'operació
   *                             <ul>"OK" = Procés executat correctament</ul>
   *                             <ul>"ERROR:" = Procés finalitzat amb error i el motiu de l'error</ul>
   */
   PROCEDURE calcular_estadistica (
      p_any                IN    despesa.anyo%TYPE,
      p_inet               IN    despesa.inet%TYPE,
      p_parlament          IN    despesa.parlament%TYPE,
      p_parlamentari       IN    despesa.parlamentari%TYPE,
      p_origenDespesa      IN    despesa.origenDespesa%TYPE,
      RSP                  OUT   NOCOPY VARCHAR2
   )
   IS
      v_nomproc	      logs.nomproc%TYPE := 'PK_ESTADISTICA.calcular_estadistica';
		v_parin		      logs.parin%TYPE;
   BEGIN
      v_parin := CONCAT('p_any=', p_any);
      v_parin := v_parin || CONCAT('; p_inet=', p_inet);
      v_parin := v_parin || CONCAT('; p_parlament=', p_parlament);
      v_parin := v_parin || CONCAT('; p_parlamentari=', p_parlamentari);
      v_parin := v_parin || CONCAT('; p_origenDespesa=', p_origenDespesa);
      
      -- Crida als procediments corresponents per calcular les estadístiques
      stat_EstTotal (p_any => p_any, p_import => p_inet, RSP => RSP);      
      stat_EstParlamentMesAlta (p_any => p_any, RSP => RSP);      
      stat_EstParlamentAnys (p_idParlament => p_parlament, RSP => RSP);
      
      -- En el cas que la despesa sigui d'un parlamentari crida als procediments
      -- corresponents per calcular les estadístiques
      IF p_origenDespesa = 2
      THEN
         stat_EstMitjanaParlamentari(p_any => p_any, RSP => RSP);
         stat_EstMitjaParlament (p_any => p_any, RSP => RSP);      
         stat_EstParlamentariAnys (p_idParlamentari => p_parlamentari, RSP => RSP);
         stat_EstParlamentParlamentari (p_any => p_any, p_idParlament => p_parlament, RSP => RSP);
         stat_EstParlamentarisOposats (p_any => p_any, RSP => RSP);
      END IF;
      p_log (v_nomproc, v_parin, RSP);
   EXCEPTION
      WHEN OTHERS
      THEN
         -- Registre l'error que li ha arribat o l'error que s'ha produit dins aquest procediment
         -- i propaga l'excepció
         IF RSP IS NULL 
         THEN
            RSP := CONCAT(PK_ERRORS.ERR_NOTRACTAT, (TO_CHAR(SQLCODE) || ' '|| SUBSTR(SQLERRM,1,400)));
         END IF;
         p_log (v_nomproc, v_parin, RSP);
         RAISE;
   END calcular_estadistica;
   
   
   PROCEDURE alta_despesa (
      p_any             IN    despesa.anyo%TYPE,
      p_inet            IN    despesa.inet%TYPE,
      p_parlament       IN    despesa.parlament%TYPE,
      p_parlamentari    IN    despesa.parlamentari%TYPE,
      p_origenDespesa   IN    despesa.origenDespesa%TYPE,
      p_estatDespesa    IN    despesa.estatDespesa%TYPE,
      RSP               OUT   NOCOPY VARCHAR2
   )
   IS
      v_nomproc	      logs.nomproc%TYPE := 'PK_ESTADISTICA.alta_despesa';
		v_parin		      logs.parin%TYPE;
      ERR_PARAMS_NULL   EXCEPTION;
   
   BEGIN
      v_parin := CONCAT('p_any=', p_any);
      v_parin := v_parin || CONCAT('; p_inet=', p_inet);
      v_parin := v_parin || CONCAT('; p_parlament=', p_parlament);
      v_parin := v_parin || CONCAT('; p_parlamentari=', p_parlamentari);
      v_parin := v_parin || CONCAT('; p_origenDespesa=', p_origenDespesa);
      v_parin := v_parin || CONCAT('; p_estatDespesa=', p_estatDespesa);
      
      -- Valida els paràmetres d'entrada
      IF p_any IS NULL OR p_inet IS NULL OR p_parlament IS NULL
         OR p_origenDespesa IS NULL OR p_estatDespesa IS NULL
         OR (p_origenDespesa = 2 AND p_parlamentari IS NULL)
      THEN
         RAISE ERR_PARAMS_NULL;
      END IF;
      
      -- realitza els càlcul de les estadístiques en el cas que la despesa 
      -- no estigui en estat Pendent d'aprovar.
      IF p_estatDespesa IN (2,3,4)
      THEN
         calcular_estadistica ( p_any           => p_any,
                                p_inet          => p_inet,
                                p_parlament     => p_parlament,
                                p_parlamentari  => p_parlamentari,
                                p_origenDespesa => p_origenDespesa,
                                RSP             => RSP
                              );
      ELSE
         RSP := 'OK';
      END IF;
      
      p_log (v_nomproc, v_parin, RSP);                           
   EXCEPTION
      WHEN ERR_PARAMS_NULL
      THEN
         RSP := PK_ERRORS.ERR_PARAMS_NULL;
         p_log (v_nomproc, v_parin, RSP);
         RAISE;
      WHEN OTHERS
      THEN
         IF RSP IS NULL 
         THEN
            RSP := CONCAT(PK_ERRORS.ERR_NOTRACTAT, (TO_CHAR(SQLCODE) || ' '|| SUBSTR(SQLERRM,1,400)));
         END IF;
         p_log (v_nomproc, v_parin, RSP);
         RAISE;
   END alta_despesa;
   
   PROCEDURE modif_despesa (
      p_anyOld             IN    despesa.anyo%TYPE,
      p_inetOld            IN    despesa.inet%TYPE,
      p_estatDespesaOld    IN    despesa.estatDespesa%TYPE,
      p_anyNew             IN    despesa.anyo%TYPE,
      p_inetNew            IN    despesa.inet%TYPE,
      p_estatDespesaNew    IN    despesa.estatDespesa%TYPE,
      p_parlament          IN    despesa.parlament%TYPE,
      p_parlamentari       IN    despesa.parlamentari%TYPE,
      p_origenDespesa      IN    despesa.origenDespesa%TYPE,
      RSP                  OUT   NOCOPY VARCHAR2
   )
   IS
      v_nomproc	      logs.nomproc%TYPE := 'PK_ESTADISTICA.modif_despesa';
		v_parin		      logs.parin%TYPE;
      ERR_PARAMS_NULL   EXCEPTION;
   BEGIN
      v_parin := CONCAT('p_anyOld=', p_anyOld);
      v_parin := v_parin || CONCAT('; p_inetOld=', p_inetOld);
      v_parin := v_parin || CONCAT('; p_estatDespesaOld=', p_estatDespesaOld);
      v_parin := v_parin || CONCAT('; p_anyNew=', p_anyNew);
      v_parin := v_parin || CONCAT('; p_inetNew=', p_inetNew);
      v_parin := v_parin || CONCAT('; p_estatDespesaNew=', p_estatDespesaNew);
      v_parin := v_parin || CONCAT('; p_parlament=', p_parlament);
      v_parin := v_parin || CONCAT('; p_parlamentari=', p_parlamentari);
      v_parin := v_parin || CONCAT('; p_origenDespesa=', p_origenDespesa);
      
      -- Valida els paràmetres d'entrada
      IF p_anyOld IS NULL OR p_inetOld IS NULL OR p_anyNew IS NULL 
         OR p_inetNew IS NULL OR p_parlament IS NULL OR p_origenDespesa IS NULL
         OR p_estatDespesaOld IS NULL OR p_estatDespesaNew IS NULL
         OR (p_origenDespesa = 2 AND p_parlamentari IS NULL)
      THEN
         RAISE ERR_PARAMS_NULL;
      END IF;
      
      IF p_estatDespesaOld != p_estatDespesaNew AND p_estatDespesaNew = 1
      THEN
      -- La despesa pasa de qualsevol estat a pendent d'aprovació per tant no ha
      -- d'entrar dins el mòdul estadístic, es tracta com si fos una baixa.
      
         calcular_estadistica ( p_any           => p_anyOld,
                                p_inet          => p_inetOld * -1,
                                p_parlament     => p_parlament,
                                p_parlamentari  => p_parlamentari,
                                p_origenDespesa => p_origenDespesa,
                                RSP             => RSP
                              );
      ELSIF p_estatDespesaOld != p_estatDespesaNew AND p_estatDespesaOld = 1
      THEN
      -- Alta de la despesa dins el mòdul estadístic ja que ha passat de 
      -- pendent d'aprovacio a un altre estat.
      
         calcular_estadistica ( p_any           => p_anyNew,
                                p_inet          => p_inetNew,
                                p_parlament     => p_parlament,
                                p_parlamentari  => p_parlamentari,
                                p_origenDespesa => p_origenDespesa,
                                RSP             => RSP
                              );
      ELSIF (p_anyOld != p_anyNew OR p_inetOld != p_inetNew) 
            AND p_estatDespesaNew IN (2,3,4)
      THEN
      -- En el cas que la despesa ja estigui dins el mòdul estadistic i es
      -- modifiqui l'any o l'import és recalculen les estadístiques.
            
         calcular_estadistica ( p_any           => p_anyOld,
                                p_inet          => p_inetOld * -1,
                                p_parlament     => p_parlament,
                                p_parlamentari  => p_parlamentari,
                                p_origenDespesa => p_origenDespesa,
                                RSP             => RSP
                              );
                              
         calcular_estadistica ( p_any           => p_anyNew,
                                p_inet          => p_inetNew,
                                p_parlament     => p_parlament,
                                p_parlamentari  => p_parlamentari,
                                p_origenDespesa => p_origenDespesa,
                                RSP             => RSP
                              );
      ELSE
         RSP := 'OK';
      END IF;
      p_log (v_nomproc, v_parin, RSP);
   EXCEPTION
      WHEN ERR_PARAMS_NULL
      THEN
         RSP := PK_ERRORS.ERR_PARAMS_NULL;
         p_log (v_nomproc, v_parin, RSP);
         RAISE;
      WHEN OTHERS
      THEN
         IF RSP IS NULL 
         THEN
            RSP := CONCAT(PK_ERRORS.ERR_NOTRACTAT, (TO_CHAR(SQLCODE) || ' '|| SUBSTR(SQLERRM,1,400)));
         END IF;
         p_log (v_nomproc, v_parin, RSP);
         RAISE;
   END modif_despesa;
   
   
   PROCEDURE baixa_despesa (
      p_any             IN    despesa.anyo%TYPE,
      p_inet            IN    despesa.inet%TYPE,
      p_parlament       IN    despesa.parlament%TYPE,
      p_parlamentari    IN    despesa.parlamentari%TYPE,
      p_origenDespesa   IN    despesa.origenDespesa%TYPE,
      p_estatDespesa    IN    despesa.estatDespesa%TYPE,
      RSP               OUT   NOCOPY VARCHAR2
   )
   IS
      v_nomproc	      logs.nomproc%TYPE := 'PK_ESTADISTICA.baixa_despesa';
		v_parin		      logs.parin%TYPE;
      ERR_PARAMS_NULL   EXCEPTION;
   BEGIN
      v_parin := CONCAT('p_any=', p_any);
      v_parin := v_parin || CONCAT('; p_inet=', p_inet);
      v_parin := v_parin || CONCAT('; p_parlament=', p_parlament);
      v_parin := v_parin || CONCAT('; p_parlamentari=', p_parlamentari);
      v_parin := v_parin || CONCAT('; p_origenDespesa=', p_origenDespesa);
      v_parin := v_parin || CONCAT('; p_estatDespesa=', p_estatDespesa);
      
      -- Valida els paràmetres d'entrada
      IF p_any IS NULL OR p_inet IS NULL OR p_parlament IS NULL
         OR p_origenDespesa IS NULL OR p_estatDespesa IS NULL
         OR (p_parlamentari IS NULL AND p_origenDespesa = 2)
      THEN
         RAISE ERR_PARAMS_NULL;
      END IF;
      
      -- En el cas que la despesa no estas pendent d'aprovar recalcula
      -- les estadístiques
      IF p_estatDespesa IN (2,3,4)
      THEN
         calcular_estadistica ( p_any           => p_any,
                                p_inet          => p_inet * -1,
                                p_parlament     => p_parlament,
                                p_parlamentari  => p_parlamentari,
                                p_origenDespesa => p_origenDespesa,
                                RSP             => RSP
                              );
      ELSE
         RSP := 'OK';
      END IF;
      p_log (v_nomproc, v_parin, RSP);
   EXCEPTION
      WHEN ERR_PARAMS_NULL
      THEN
         RSP := PK_ERRORS.ERR_PARAMS_NULL;
         p_log (v_nomproc, v_parin, RSP);
         RAISE;
      WHEN OTHERS
      THEN
         IF RSP IS NULL 
         THEN
            RSP := CONCAT(PK_ERRORS.ERR_NOTRACTAT, (TO_CHAR(SQLCODE) || ' '|| SUBSTR(SQLERRM,1,400)));
         END IF;
         p_log (v_nomproc, v_parin, RSP);
         RAISE;
   END baixa_despesa;
END;
/
