CREATE OR REPLACE PACKAGE PK_ESTADISTICA 
AS
   /**
   * <p>
   * <b>Project:</b>         Control Despesa<br/>
   * <b>Description:</b>     Paquet encarregat de fer els càlculs de les estadśitiques<br/>
   * <b>DB impact:</b>       SI<br/>
   * <b>Commit inside:</b>   NO<br/>
   * <b>Rollback inside:</b> NO<br/>
   * <b>Autonomous Transaction inside:</b> NO<br/>
   * </p>
   * @author Toni Torres
   * @headcom
   */
   
   /**
   * Alta despesa dins les estadístiques
   * <p>Aquest procediment és cridada quan es dona d'alta una nova despesa,
   * ja sigui associada a un parlament o a un parlamentari.</p>
   * <p>Aquesta crida als procediments interns d'aquest paquet per actualitzar
   * les dades de les estadístiques<p>
   *
   * @param p_any                NUMBER. Any de la despesa
   * @param p_inet               NUMBER. Import net de la despesa
   * @param p_parlament          NUMBER. Codi identificatiu del parlament
   * @param p_parlamentari       NUMBER. Codi identificatiu del parlamentari
   * @param p_origenDespesa      NUMBER. Codi que identifica si la despesa és del parlament o parlamentari
   * @param p_estatDespesa       NUMBER. Codi estat de la despesa
   * @param RSP                  Resposta de l'operació
   *                             <ul>"OK" = Procés executat correctament</ul>
   *                             <ul>"ERROR:" = Procés finalitzat amb error i el motiu de l'error</ul>
   *
   * @throws                     ERR_PARAMS_NULL                  ERROR: HI HA CAMPS NO INFORMATS   
   * @throws                     ERR_STAT_ESTTOTAL                ERROR NO TRACTAT ESTADISTICA (ESTTOTAL): codiError + literalError
   * @throws                     ERR_STAT_ESTMITJAPARLAMENT       ERROR NO TRACTAT ESTADISTICA (ESTMITJAPARLAMENT): codiError + literalError
   * @throws                     ERR_STAT_ESTPARMENTARISOPOSATS   ERROR NO TRACTAT ESTADISTICA (ESTPARLAMENTARISOPOSATS): codiError + literalError
   * @throws                     ERR_STAT_ESTMITJANAPARMENTARI    ERROR NO TRACTAT ESTADISTICA (ESTMITJANAPARLAMENTARI): codiError + literalError
   * @throws                     ERR_STAT_ESTPARLAMENTARIANYS     ERROR NO TRACTAT ESTADISTICA (ESTPARLAMENTARIANYS): codiError + literalError
   * @throws                     ERR_STAT_ESTPARLAMENTMESALTA     ERROR NO TRACTAT ESTADISTICA (ESTPARLAMENTMESALTA): codiError + literalError
   * @throws                     ERR_STAT_ESTPARMENTPARMENTARI    ERROR NO TRACTAT ESTADISTICA (ESTPARLAMENTPARLAMENTARI): codiError + literalError
   * @throws                     ERR_STAT_ESTPARLAMENTANYS        ERROR NO TRACTAT ESTADISTICA (ESTPARLAMENTANYS): codiError + literalError   
   * @throws                     ERR_NOTRACTAT                    ERROR NO TRACTAT: codiError + literalError
   */
   PROCEDURE alta_despesa (
      p_any             IN    despesa.anyo%TYPE,
      p_inet            IN    despesa.inet%TYPE,
      p_parlament       IN    despesa.parlament%TYPE,
      p_parlamentari    IN    despesa.parlamentari%TYPE,
      p_origenDespesa   IN    despesa.origenDespesa%TYPE,
      p_estatDespesa    IN    despesa.estatDespesa%TYPE,
      RSP               OUT   NOCOPY VARCHAR2
   );
   
   /**
   * Modifica les dades de la despesa dins les estadístiques
   * <p>Aquest procediment és cridat quan es dona es modifica una despesa,
   * ja sigui associada a un parlament o a un parlamentari. Abans de realitzar
   * les modificacions dins les taules corresponents comprova que s'ha modificat
   * l'any o l'import net de la despesa, en cas contrari no realitza cap operació</p>
   * <p>Aquesta crida als procediments interns d'aquest paquet per actualitzar
   * les dades de les estadístiques<p>
   *
   * @param p_anyOld             NUMBER. Anterior any de la despesa
   * @param p_inetOld            NUMBER. Anterior import net de la despesa
   * @param p_estatDespesaOld    NUMBER. Anterior estat de la despesa
   * @param p_anyNew             NUMBER. Nou any de la despesa
   * @param p_inetNew            NUMBER. Nou import net de la despesa
   * @param p_estatDespesaNew    NUMBER. Nou estat de la despesa
   * @param p_parlament          NUMBER. Codi identificatiu del parlament
   * @param p_parlamentari       NUMBER. Codi identificatiu del parlamentari
   * @param p_origenDespesa      NUMBER. Codi que identifica si la despesa és del parlament o parlamentari
   * @param RSP                  Resposta de l'operació
   *                             <ul>"OK" = Procés executat correctament</ul>
   *                             <ul>"ERROR:" = Procés finalitzat amb error i el motiu de l'error</ul>
   *
   * @throws                     ERR_PARAMS_NULL                  ERROR: HI HA CAMPS NO INFORMATS   
   * @throws                     ERR_STAT_ESTTOTAL                ERROR NO TRACTAT ESTADISTICA (ESTTOTAL): codiError + literalError
   * @throws                     ERR_STAT_ESTMITJAPARLAMENT       ERROR NO TRACTAT ESTADISTICA (ESTMITJAPARLAMENT): codiError + literalError
   * @throws                     ERR_STAT_ESTPARMENTARISOPOSATS   ERROR NO TRACTAT ESTADISTICA (ESTPARLAMENTARISOPOSATS): codiError + literalError
   * @throws                     ERR_STAT_ESTMITJANAPARMENTARI    ERROR NO TRACTAT ESTADISTICA (ESTMITJANAPARLAMENTARI): codiError + literalError
   * @throws                     ERR_STAT_ESTPARLAMENTARIANYS     ERROR NO TRACTAT ESTADISTICA (ESTPARLAMENTARIANYS): codiError + literalError
   * @throws                     ERR_STAT_ESTPARLAMENTMESALTA     ERROR NO TRACTAT ESTADISTICA (ESTPARLAMENTMESALTA): codiError + literalError
   * @throws                     ERR_STAT_ESTPARMENTPARMENTARI    ERROR NO TRACTAT ESTADISTICA (ESTPARLAMENTPARLAMENTARI): codiError + literalError
   * @throws                     ERR_STAT_ESTPARLAMENTANYS        ERROR NO TRACTAT ESTADISTICA (ESTPARLAMENTANYS): codiError + literalError
   * @throws                     ERR_NOTRACTAT                    ERROR NO TRACTAT: codiError + literalError   
   */
   PROCEDURE modif_despesa (
      p_anyOld          IN    despesa.anyo%TYPE,
      p_inetOld         IN    despesa.inet%TYPE,
      p_estatDespesaOld IN    despesa.estatDespesa%TYPE,
      p_anyNew          IN    despesa.anyo%TYPE,
      p_inetNew         IN    despesa.inet%TYPE,
      p_estatDespesaNew IN    despesa.estatDespesa%TYPE,
      p_parlament       IN    despesa.parlament%TYPE,
      p_parlamentari    IN    despesa.parlamentari%TYPE,
      p_origenDespesa   IN    despesa.origenDespesa%TYPE,
      RSP               OUT   NOCOPY VARCHAR2
   );
   
   /**
   * Dona de baixa la despesa dins les estadístiques
   * <p>Aquest procediment és cridat quan es dona es dona de baixa una despesa,
   * ja sigui associada a un parlament o a un parlamentari.</p>
   * <p>Aquesta crida als procediments interns d'aquest paquet per actualitzar
   * les dades de les estadístiques<p>
   *
   * @param p_anyNew             NUMBER. Nou any de la despesa
   * @param p_inetNew            NUMBER. Nou import net de la despesa
   * @param p_parlament          NUMBER. Codi identificatiu del parlament
   * @param p_parlamentari       NUMBER. Codi identificatiu del parlamentari
   * @param p_origenDespesa      NUMBER. Codi que identifica si la despesa és del parlament o parlamentari
   * @param p_estatDespesa       NUMBER. Codi estat de la despesa.
   * @param RSP                  Resposta de l'operació
   *                             <ul>"OK" = Procés executat correctament</ul>
   *                             <ul>"ERROR:" = Procés finalitzat amb error i el motiu de l'error</ul>
   *
   * @throws                     ERR_PARAMS_NULL                  ERROR: HI HA CAMPS NO INFORMATS
   * @throws                     ERR_STAT_ESTTOTAL                ERROR NO TRACTAT ESTADISTICA (ESTTOTAL): codiError + literalError
   * @throws                     ERR_STAT_ESTMITJAPARLAMENT       ERROR NO TRACTAT ESTADISTICA (ESTMITJAPARLAMENT): codiError + literalError
   * @throws                     ERR_STAT_ESTPARMENTARISOPOSATS   ERROR NO TRACTAT ESTADISTICA (ESTPARLAMENTARISOPOSATS): codiError + literalError
   * @throws                     ERR_STAT_ESTMITJANAPARMENTARI    ERROR NO TRACTAT ESTADISTICA (ESTMITJANAPARLAMENTARI): codiError + literalError
   * @throws                     ERR_STAT_ESTPARLAMENTARIANYS     ERROR NO TRACTAT ESTADISTICA (ESTPARLAMENTARIANYS): codiError + literalError
   * @throws                     ERR_STAT_ESTPARLAMENTMESALTA     ERROR NO TRACTAT ESTADISTICA (ESTPARLAMENTMESALTA): codiError + literalError
   * @throws                     ERR_STAT_ESTPARMENTPARMENTARI    ERROR NO TRACTAT ESTADISTICA (ESTPARLAMENTPARLAMENTARI): codiError + literalError
   * @throws                     ERR_STAT_ESTPARLAMENTANYS        ERROR NO TRACTAT ESTADISTICA (ESTPARLAMENTANYS): codiError + literalError
   * @throws                     ERR_NOTRACTAT                    ERROR NO TRACTAT: codiError + literalError   
   */
   PROCEDURE baixa_despesa (
      p_any             IN    despesa.anyo%TYPE,
      p_inet            IN    despesa.inet%TYPE,
      p_parlament       IN    despesa.parlament%TYPE,
      p_parlamentari    IN    despesa.parlamentari%TYPE,
      p_origenDespesa   IN    despesa.origenDespesa%TYPE,
      p_estatDespesa    IN    despesa.estatDespesa%TYPE,
      RSP               OUT   NOCOPY VARCHAR2
   );   
      
END;
/
